/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * This class is an encapsulation of all methods and data structures that are responsible for 
 *  dealing with the instrument plugins to set or read definitions from the instrument plug-ins
 * ------------------------------------------------------------------
 */

/*
 **************************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =============================================================================================
 *   Venkatachalam  09/05/2011  Updated for concurrent test modifications.
 *                              1)Modified CreatePinTable() method. 
 *   Ananda Kumary  01/18/2010  Fixed tersw00152003 - disabled the sort propery of ITCOStrings data type 
 *                              in GetChoices(). 
 *   Kranthi Bharath 10/20/09   Fixed the defect tersw00148903
 *   Kannan K       08/25/2009  Added the Dynamic LoadSettings button name feature.
 *   Suresh V A     04/23/2008  Fixed tersw00120161. 
 *   Kiran          11/1/2007   added piCache dictionary for performance issue
 *   Andy Kittross 10/22/2007   fixed crash using String.Remove(0) on empty string (tersw00113020)
 *                              And also tersw00113032 (handling of blanks in pin CSL)
 *   M.Raykin      10/5/2007    fixed tersw00112398
 *   P.Babu         9/9/2007    Made GetInstrumentInterface as a public method  
 *   Nadeem Khan    8/7/2007    Created.
 **************************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Teradyne.Igxl.PSetSignalsEditor;
using PARAMETERCOLLECTIONINTERFACELib;
using Teradyne.Igxl.PSetSignalsEditor.PSetSignalsEditorServices;
using Teradyne.Igxl.PSetSignalsEditor.Exceptions;
using System.Windows.Forms;
using Teradyne.Igxl.PSetSignalsEditor.BalloonTip;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    /// <summary>
    /// This class is an encapsulation of all methods and data structures that are responsible for 
    /// dealing with the instrument plugins to set or read definitions from the instrument plug-ins
    /// </summary>
    public class InstrumentEngine : IDisposable
    {
        bool is_disposed = false;
        CollectionEditHelperLib.CCollectionEditHelper m_editHelper; //Instance of collection edit helper that can point to either 
        //a PSet/Signal helper depending upon the mode in which the editor is in.
        CollectionEditHelperLib.CCollectionEditHelper m_psetHelper;
        CollectionEditHelperLib.CCollectionEditHelper m_signalHelper;

        Dictionary<String, String> piCache; // Dictionary used for caching the Physical instrument
        // of a particular pinname, Logical Instrument
        // Key is a combination of pinname and Logical Instrument Name.

        GlobalSettings m_GlobaSettings;

        /// <summary>
        /// This member will hold the reference of frmPSetSignalsEditor to get tool id as well TDe object
        /// </summary>
        private frmPSetSignalsEditor m_frmPSetSignalsEditor = null;

        public InstrumentEngine(frmPSetSignalsEditor frmPSetSignalsEditor, ref CollectionEditHelperLib.CCollectionEditHelper pHelper, ref CollectionEditHelperLib.CCollectionEditHelper sHelper, ref GlobalSettings glblSettings)
        {
            m_frmPSetSignalsEditor = frmPSetSignalsEditor;
            if (pHelper == null || sHelper == null) throw new ArgumentNullException("One or more arguments passed to constructor are null");
            m_psetHelper = pHelper;
            m_signalHelper = sHelper;
            m_GlobaSettings = glblSettings;
            UpdateHelper(m_GlobaSettings.MainPanel.NSPCP_MODE);
            piCache = new Dictionary<string, String>();
        }

        /// <summary>
        /// This method is used for clearing the piCache dictionary's cache.
        /// </summary>
        public void ClearPhysicalInstCache()
        {
            piCache.Clear();
        }
        protected virtual void Dispose(bool disposing)
        {
            if (!is_disposed)
            {
                if (disposing)
                {
                    m_editHelper = null;
                    m_psetHelper = null;
                    m_signalHelper = null;
                }
            }
            this.is_disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            //tell the garbage collector not to finalize
            GC.SuppressFinalize(this);
        }


        ~InstrumentEngine()
        {
            Dispose(false);
            piCache.Clear();
            piCache = null;
        }

        public void UpdateHelper(GlobalUtils.NspcpType currentType)
        {
            //Update Collection Edit Helper depending upon what the currently selected nspcp type is.
            if (currentType == GlobalUtils.NspcpType.pset)
            {
                m_editHelper = m_psetHelper;
            }
            else
            {
                m_editHelper = m_signalHelper;
            }
        }

        /// <summary>
        /// Retrieve the reference to the interface to an instrument plugin.
        /// </summary>
        /// <param name="instrName">Logical Instrument Name</param>
        /// <returns>Parameter Collection Interface</returns>
        public IParameterCollection GetInstrumentInterface(String instrName)
        {
            if (instrName == null) throw new ArgumentNullException("One of more arguments passed is null");

            try
            {
                IParameterCollection pColl = (PARAMETERCOLLECTIONINTERFACELib.IParameterCollection)m_editHelper.get_Instrument(instrName);
                return pColl;
            }
            catch (Exception exp)
            {
                throw new CollectionEditHelperException("Error in CollectionEditHelper while receiving instrument interface" + exp.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }
        }

        private String GetPhysicalInstrument(String pinName, String instrName)
        {
            String phyInstr;

            if (pinName == null || instrName == null) throw new ArgumentNullException("One or more arguments passed to GetPhysicalInstruments is null");

            try
            {
                // the following if--else block code has been added to improve the 
                // performance when populating grid. 
                // Here The physical instrument has been cahed.
                String sKey = String.Concat(pinName, instrName);
                if (piCache.ContainsKey(sKey))
                {
                    phyInstr = piCache[sKey];
                }
                else
                {
                    phyInstr = m_editHelper.get_BoardName(pinName, instrName);
                    piCache.Add(sKey, phyInstr);
                }
            }
            catch (Exception exp)
            {
                throw new CollectionEditHelperException("Error retrieving physical instrument name " + exp.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }
            return phyInstr;
        }

        ///<summary>
        ///Validates whether the pins in a given pinSpec are valid or not.
        ///</summary>
        ///<param name="pinSpec"></param>
        ///<returns></returns>
        public Boolean ValidateNSPCP(String pinSpec, String Instr, String nspcpName, ref String error)
        {

            //Check if pinspec is valid
            if (!CheckPin(ref pinSpec))
            {
                error = "Selected pin name is invalid";
                return false;
            }

            //Check if instrument is valid
            if (!CheckInstrument(Instr, true, pinSpec))
            {
                error = "Selected instrument is invalid or does not map to the selected pin";
                return false;
            }

            //Finally check if the nspcp is valid for the given combination
            List<String> validNspcp;
            List<String> errors = new List<string>();
            validNspcp = GetList(pinSpec, Instr, ref errors);

            foreach (String vnspcp in validNspcp)
            {
                if (vnspcp.ToUpper().Equals(nspcpName.ToUpper()))
                {
                    return true;
                }
            }
            error = "The PSet/Signal name provided is not valid for the given pin and instrument name";
            return false;
        }

        /// <summary>
        /// Checks if the pin spec specified is valid or not.
        /// i.e. if it exists in the pin map.
        /// </summary>
        /// <param name="pinSpec"></param>
        /// <returns>returns true if pinspec is valid, otherwise returns false</returns>
        public Boolean CheckPin(ref String pinSpec)
        {
            List<string> AllPins;
            if (pinSpec == null) throw new ArgumentNullException("One of more arguments passed to CheckPin are null");

            AllPins = RetrievePinList();

            if (AllPins == null) throw new NullReferenceException("Complete pinlist retrieved was null");

            //Check if all the pin or pin groups in the pin spec are valid or not.
            String[] pinList = pinSpec.Split(',');

            if (pinList == null) throw new NullReferenceException("Decomposed list retrieved was null");

            Boolean validPin;

            for (int i = 0; i < pinList.Length; i++)
            {
                if (!pinList[i].Equals(""))
                {
                    String pinListUpper = pinList[i].ToUpper();
                    validPin = false;
                    for (int j = 0; j < AllPins.Count; j++)
                    {
                        if (AllPins[j].ToUpper().Equals(pinListUpper))
                        {
                            if (DoesPinNameExistsPinMapSheet(pinList[i]))
                            {
                            validPin = true;
                            // fix "case" problem (tersw00112398)
                            pinList[i] = AllPins[j];
                            break;
                        }
                    }
                    }
                    if (!validPin)
                    {
                        return false;
                    }
                }
            }
            // Clear pinSpec and recreate it with correctly capitalized pins (tersw00112398)
            // And leave out any blank pins in a CSL!
            pinSpec = System.String.Empty;
            for (int i = 0; i < pinList.Length; i++)
            {
                if (!pinList[i].Equals(""))
                {
                    pinSpec += pinList[i];
                    // Add a comma unless this is the last entry or the next entry is blank
                    if ((i < pinList.Length - 1) && !(pinList[i + 1].Equals(System.String.Empty)))
                        pinSpec += ',';
                }
            }
            return true;
        }

        /// <summary>
        /// Finds if the pinName is in the PinMap sheet Collection
        /// </summary>
        /// <param name="pinName">PinName collection</param>
        /// <returns>returns true if DMPinMapEntry is not equal to null , otherwise returns false</returns>
        private bool DoesPinNameExistsPinMapSheet(string pinName)
        {
            if (!String.IsNullOrEmpty(pinName))
            {
                //Find the pinName in the PinMap
                DMGRLib.DMPinMapEntry pinEntry = MyServices.DmgrServiceCOM.PinMapEntries.Find(pinName);
                if (pinEntry != null)
                    return true;              
            }
            return false;
        }

        private List<String> RetrievePinList()
        {
            List<string> AllPins;
            DMGRLib.ITCOStrings str;

            try
            {
                //Get the list of all pins on the pin map.
                str = MyServices.DmgrServiceCOM.ReferenceMgr.GetReferenceList(GlobalUtils.KEYSET_NAME, "");
            }
            catch (Exception e)
            {
                throw new ServicesException("Error in DMGR while retrieving the list pins" + e.Message);
            }

            if (str == null) throw new Exception("Error Receiving list of pins from Reference Manager");

            AllPins = CreateList((TERCOMMONOBJLib.ITCOStrings)str);

            return AllPins;
        }

        /// <summary>
        /// Checks if the instrument name specificied is valid. Additionally, also
        /// checks if the instrument is mapped to the specified pin in the channel
        /// map.
        /// </summary>
        /// <param name="Instr"></param>
        /// <param name="usePin">To check if instrument is mapped to a given pin</param>
        /// <param name="pinName"></param>
        /// <returns></returns>
        public Boolean CheckInstrument(String Instr, Boolean usePin, String pinName)
        {
            //Get all instruments, this is achieved by passing an empty value to 
            //the GetInstruments method.
            String param;

            if (Instr == null || pinName == null) throw new ArgumentNullException("One of the arguments supplied is null");

            if (usePin)
            {
                param = pinName;
            }
            else
            {
                param = GlobalUtils.EMPTY_VALUE;
            }

            String[] AllInstr = GetInstruments(param);

            foreach (String inst in AllInstr)
            {
                if (inst.ToUpper().Equals(Instr.ToUpper()))
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Rebuilds the keyset (pin/instrument data)
        /// </summary>
        /// <param name="instrument"></param>
        /// <returns></returns>
        public void Rebuild()
        {
            m_editHelper.RebuildKeySet();
        }

        /// <summary>
        /// Return the list of pins and pin groups associated 
        /// with the given instrument.
        /// </summary>
        /// <param name="instrument"></param>
        /// <returns></returns>
        public Dictionary<String, String> CreatePinTable()
        {
            DMGRLib.ITCOStrings str;
            List<string> pinList = new List<string>();
            Dictionary<String, String> PinTable = new Dictionary<String, String>();

            try
            {
                str = MyServices.DmgrServiceCOM.ReferenceMgr.GetReferenceList(GlobalUtils.KEYSET_NAME, "");
            }
            catch (Exception e)
            {
                throw new ServicesException("Error in DMGR while retrieving the list pins" + e.Message);
            }

            if (str == null) throw new NullReferenceException("List of pins received was null");

            pinList = CreateList((TERCOMMONOBJLib.ITCOStrings)str);

            Array pinListArr = pinList.ToArray();
            //set the current flowdomain value which was selected by user in flowdomain selector
            m_frmPSetSignalsEditor.TDE.SetToolFlowDomain();

            // Get the pins associated with the current flow domain
            pinListArr = m_frmPSetSignalsEditor.TDE.FilterPinListToFlowDomain(ref pinListArr);
            
            //Create the Pin->Instrument Map.
            foreach (String s in pinListArr)
            {
                String[] instr = GetInstruments(s);
                string InstrList = String.Join(",", instr);
                if (!string.IsNullOrEmpty(InstrList))
                    PinTable.Add(s, InstrList);
            }

            return PinTable;
        }

        /// <summary>
        /// This method converts an enumeration of strings into a list.
        /// </summary>
        /// <returns></returns>
        private static List<string> CreateList(TERCOMMONOBJLib.ITCOStrings str)
        {
            List<string> strList;
            strList = new List<string>();
            if (str != null)
            {
                foreach (string s in str)
                {
                    strList.Add(s);
                }
            }
            else
            {
                throw new ArgumentNullException("Null Arguments passed");
            }
            return strList;
        }

        /// <summary>
        /// This method converts an ITCOBSTR into a string.
        /// </summary>
        /// <returns></returns>
        private static List<String> CreateListfromBSTR(TERCOMMONOBJLib.ITCOBSTRs str)
        {
            List<String> strList;
            strList = new List<String>();
            if (str != null)
            {
                foreach (string s in str)
                {
                    strList.Add(s);
                }
            }
            else
            {
                throw new ArgumentNullException("Null Arguments passed");
            }
            return strList;
        }

        /// <summary>
        /// This method returns the definition of a PSet/Signal using the ParameterValues 
        /// method of the IParameterCollection interface. It populates the nspcpItem object 
        /// with the definition it retrieves from the instrument plug-in.
        /// </summary>
        /// <returns> NSPCP Item object which is initialized with the definition retrieved from the plugins.</param>
        /// <param name="pColl">Reference to the ParameterCollection Interface Implmentation for that particular plugin.</param>
        public NspcpItem GetNSPCPDefiniton(String pinName, String instrName, String nspcpName, Teradyne.Igxl.PSetSignalsEditor.GlobalUtils.NspcpType type, ref List<String> errors)
        {
            if (pinName == null || instrName == null || nspcpName == null)
            {
                throw new ArgumentNullException("Some or all arguments passed are null");
            }

            ITCOBSTRs names;
            ITCOBSTRs values;
            ITCOBSTRs outErrorInfo;
            ITCOBSTRs outErrorNames;
            String phyInstr = "";
            Array outenables;
            Boolean[] enables;
            Dictionary<String, String> nvp = new Dictionary<string, string>();

            enables = new Boolean[0];
            outenables = enables as Array;
            names = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            values = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            outErrorInfo = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            outErrorNames = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());



            //Get the instrument interface
            IParameterCollection pColl = GetInstrumentInterface(instrName);

            if (pColl == null) throw new CollectionEditHelperException("Parameter Collection could not be retrieved for instrument " + instrName);

            if (m_editHelper == null) throw new NullReferenceException("CollectionEditHelper instance is null");

            //Get Physical Instrument
            phyInstr = GetPhysicalInstrument(pinName, instrName);

            // The instruments which implements IParameterCollection.Configure() might reset the flow domains
            // hence lock the currently selected flow domain in flow domain selector of tool manager/ tde flow domain
            // selector
            using (new PsetAndSignalEditorFlowDomainUtil(m_frmPSetSignalsEditor.TMtoolID))
            {
                pColl.Configure(ref names, ref values, phyInstr, pinName, ref outenables);
            }

            if (names.Count < 1)
            {
                throw new Exception("Configure method failed to get the list of parameter names for the particular instrument");
            }

            try
            {
                // The instruments which implements IParameterCollection.ParameterValues() might reset the flow domains
                // hence lock the currently selected flow domain in flow domain selector of tool manager/ tde flow domain
                // selector
                using (new PsetAndSignalEditorFlowDomainUtil(m_frmPSetSignalsEditor.TMtoolID))
                {
                    //Get Definition
                    pColl.ParameterValues(ref names, ref values, phyInstr, nspcpName, pinName, ref outErrorNames, ref outErrorInfo);
                }
            }
            catch (Exception e)
            {
                throw new Exception("Error receiving parameter values from plugins " + e.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }


            if (outErrorInfo.Count > 0)
            {
                errors = CreateListfromBSTR((TERCOMMONOBJLib.ITCOBSTRs)outErrorInfo);
                return null;  //This will be handled
            }
            else
            {
                if (names.Count == values.Count)
                {
                    //Create Dictionary
                    for (int i = 1; i <= names.Count; i++)
                    {
                        nvp.Add(names[i], values[i]);
                    }
                }
                else
                {
                    throw new NullReferenceException("Incorrect size of name-value pairs received ");
                }

                NspcpItem tmpObj = new NspcpItem(type, nspcpName, pinName, instrName, phyInstr, names.Count, nvp);
                return tmpObj;
            }
        }


        /// <summary>
        /// This method returns the definition of a given nspcp from the instrument driver.
        /// </summary>
        /// <param name="pinName">Pin to which the nspcp is mapped</param>
        /// <param name="instrName">Instrument type of the nspcp</param>
        /// <param name="nspcpName">Name of the nspcp</param>
        /// <param name="type">PSet/Signal</param>
        /// <returns>Object containing the definition.</returns>
        public NspcpItem GetDefiniton(String pinName, String instrName, String nspcpName, Teradyne.Igxl.PSetSignalsEditor.GlobalUtils.NspcpType type, ref List<String> errorInfo)
        {

            if (pinName == null || instrName == null || nspcpName == null)
            {
                throw new ArgumentNullException("One or more arguments passed to GetDefinition is null");
            }

            List<string> errors = new List<string>();

            //Decompose to check if its a pin list or a pin group.
            String[] pinList = DecomposePinList(pinName);

            if (pinList == null) throw new NullReferenceException("Decomposed list retrieved was null");

            List<NspcpItem> objList = new List<NspcpItem>();
            NspcpItem obj;

            if (pinList.Length > 0)        //Check if we actually have some pins in our pinspec
            {
                foreach (String pin in pinList) //Go through all pins in the pinspec
                {
                    obj = GetNSPCPDefiniton(pin, instrName, nspcpName, type, ref errors); //Get their definition  
                    if (obj == null)    //Null object returned, must be errors while retrieving definitions.
                    {
                        errorInfo.AddRange(errors);
                        return null;
                    }
                    objList.Add(obj); //Add them to a definition list
                }

                if (IsEqualDefinition(objList)) //Check if all definitions across the pin list/groups are consistent...
                {
                    NspcpItem tmpObj = objList[GlobalUtils.FIRST_ELEMENT];   //If so, retrieve any one of those definitions (doesnt matter which) 
                    tmpObj.pinName = pinName;      //Update the name to the group/list name
                    return tmpObj; //return back to the main UI.
                }
                else
                {
                    throw new IncompatibleNSPCPException("The PSet/Signal that you selected does not have the same definition across all pins in the pingroup/pinlist specified.");
                }
            }
            else
            {
                throw new Exception("Cannot Retrieve Definition, No Pins Present in PinSpec");
            }

        }

        /// <summary>
        /// This method sets the definition of PSet/Signal using the Validate method of the IParameterCollection Interface.
        /// It extracts the definition from the nspcpItem object being passed.
        /// All errors are set in the errParams and the errInfo arrays.
        /// </summary>
        /// <param name="obj">NSPCP information is extracted from this object</param>
        /// <param name="pColl">This interface is used to invoke the validate method on the plugin</param>
        /// <returns> The count of validation errors is returned </returns>
        /// <param name="errParam">The parameter names where errors have ocurred</param>
        /// <param name="errInfo">The error Information</param>
        public void SetDefinition(ref Dictionary<String, NspcpItem> objList)
        {
            if (objList == null) throw new ArgumentNullException("One or more arguments passed to SetDefinition is null");

            if (objList.Count > 0)
            {
                foreach (String key in objList.Keys)
                {
                    NspcpItem obj = objList[key];
                    SetDefinition(ref obj);
                }
            }
        }


        /// <summary>
        /// This methods takes an nspcp item object containing the definition for an nspcp
        /// and sets it value in the driver.
        /// </summary>
        /// <param name="obj">Object containing definition for the NSPCP</param>
        public void SetDefinition(ref NspcpItem obj)
        {
            if (obj == null) throw new ArgumentNullException("One or more arguments passed to SetDefinition are null");
            int idx;
            ITCOBSTRs names;
            ITCOBSTRs values;
            ITCOBSTRs outErrorInfo;
            ITCOBSTRs outErrorNames;
            names = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            values = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            outErrorInfo = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            outErrorNames = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());

            //Get the instrument interface
            IParameterCollection pColl = GetInstrumentInterface(obj.propLogicalInstrument);
            //Clear the name value pair for the next nspcp
            names.Clear();
            values.Clear();
            idx = 1;

            //Get each ParamName/Value association
            foreach (KeyValuePair<String, String> kvp in obj.NameValuePairs)
            {
                //Populate the ITCOBSTRs
                names.Add(kvp.Key, idx);
                values.Add(kvp.Value, idx);
                idx++;
            }

            try
            {
                // The instruments which implements IParameterCollection.Validate() might reset the flow domains
                // hence lock the currently selected flow domain in flow domain selector of tool manager/ tde flow domain
                // selector
                using (new PsetAndSignalEditorFlowDomainUtil(m_frmPSetSignalsEditor.TMtoolID))
                {
                    //Validate the name value pair
                    pColl.Validate(ref names, ref values, obj.propPhysicalInstrument, obj.propName, obj.pinName, ref outErrorNames, ref outErrorInfo);
                }
            }
            catch (Exception exp)
            {
                throw new NotImplementedException("Error occurred while validating PSet/Signals " + exp.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }

            //Clear existing errors
            obj.propErrInfo.Clear();
            obj.propErrName.Clear();

            //Store validation errors (if any)
            if (outErrorInfo.Count > 0)
            {
                obj.propErrInfo = CreateListfromBSTR((TERCOMMONOBJLib.ITCOBSTRs)outErrorInfo);
                obj.propErrName = CreateListfromBSTR((TERCOMMONOBJLib.ITCOBSTRs)outErrorNames);
            }
        }

        /// <summary>
        /// This method retrieves the list of PSets/Signals defined for a given pinName and Instrument Name
        /// using the PSetSignalList method of the IParameterCollection interface.
        /// </summary>
        /// <param name="pinName">Name of PinSpec</param>
        /// <param name="instrName">Logical Instrument</param>
        public List<String> GetList(String pinName, String instrName, ref List<String> errors)
        {
            if (pinName == null || instrName == null || errors == null) throw new ArgumentNullException("One or more arguments passed to GetList is NULL");
            List<String> retList = new List<string>();
            List<List<String>> bigList = new List<List<string>>();

            //Decompose the PinList/Group
            String[] pinList = DecomposePinList(pinName);

            if (pinList == null) throw new NullReferenceException("Error while receiving decomposed pin list value");

            //List creation step.
            foreach (String pin in pinList)
            {
                List<String> nspcpList = new List<String>();
                nspcpList = GetNSPCPList(pin, instrName, ref errors);

                //Error in retrieving list.
                if (nspcpList == null) return null;

                //Get the list of nspcps for this pin
                bigList.Add(nspcpList);
            }

            if (bigList.Count > 1)
            {
                //Get all the list of nspcp names that are shared across the pins present in the pinlist/group provided
                retList = GetCommonElements(bigList);
            }
            else
            {
                retList = bigList[GlobalUtils.FIRST_ELEMENT];
            }

            return retList;
        }

        /// <summary>
        /// Decomposes a Pin Spec into a list of pin names that compose that pinspec.
        /// </summary>
        /// <param name="pinspec">PinSpec Name</param>
        /// <returns>List of pins</returns>
        private String[] DecomposePinList(String pinspec)
        {

            if (pinspec == null) throw new ArgumentNullException("One of more arguments passed to DecomposePinList is null");
            Array arrPinList;
            String[] strPinList;
            int pinCount;

            //TypeCast Array to String[] so that it can be passed as a parameter
            //to the DecomposePinList method.
            strPinList = new String[0];
            arrPinList = strPinList as Array;

            try
            {
                if (m_editHelper.IsValidPinSpec(pinspec))
                    //Decompose the pin spec to get the list of pins.
                    MyServices.DmgrServiceCOM.DecomposePinList(pinspec, out arrPinList, out pinCount);
            }
            catch (Exception e)
            {
                throw new ServicesException("Error in DMGR Service while Decomposing Pin List" + e.Message);
            }

            return ((String[])arrPinList);
        }


        /// <summary>
        /// This method is used to extract the list of the names of PSets/Signals for a given 
        /// Pin-Instrument combination.
        /// </summary>
        /// <param name="pinName">Pin Name</param>
        /// <param name="instrName">Instrument Name</param>
        /// <returns>List of names of PSets/Signals</returns>
        private List<String> GetNSPCPList(String pinName, String instrName, ref List<String> errInfo)
        {
            if (pinName == null || instrName == null || errInfo == null) throw new ArgumentNullException("One or more arguments passed to GetNSPCPList are null");

            try
            {
                ITCOBSTRs outErrorInfo;
                ITCOBSTRs outErrorNames;
                String list = "";       //Initialize the string
                outErrorInfo = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
                outErrorNames = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
                IParameterCollection pColl = GetInstrumentInterface(instrName);
                using (new PsetAndSignalEditorFlowDomainUtil(m_frmPSetSignalsEditor.TMtoolID))
                {

                list = pColl.PSetSignalList(pinName, ref outErrorNames, ref outErrorInfo);
                }

                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;

                List<String> newList = new List<string>();

                if (list != null)
                {
                    String[] nspcpList = list.Split(',');
                    foreach (String element in nspcpList)  //Convert everything to lower-case.
                    {
                        newList.Add(element.ToLower());
                    }
                }
                else
                {
                    errInfo = CreateListfromBSTR((TERCOMMONOBJLib.ITCOBSTRs)outErrorInfo);
                    return null;
                }

                return newList;
            }
            catch (Exception exp)
            {
                throw new NotImplementedException("Cannot Retreive PSet/Signal List " + exp.StackTrace);
            }
        }

        /// <summary>
        /// Helper routine which takes multiple lists of elements
        /// and returns a list which contains elements common across
        /// all those lists.
        /// </summary>
        /// <param name="ListsToMerge">Collection of lists to merge</param>
        /// <returns>Final merged list with common elements</returns>
        List<string> GetCommonElements(List<List<String>> ListsToMerge)
        {
            List<string> MergedList = new List<string>();
            if (ListsToMerge.Count > 0)
            {
                List<String> firstList = ListsToMerge[0];
                Boolean keep = true;
                foreach (String s in firstList)//Go through each element in first list
                {
                    keep = true;
                    foreach (List<String> memList in ListsToMerge)
                    {
                        if (!memList.Contains(s)) { keep = false; break; }//if the list does not contain on of the elements, dicard that element..
                    }
                    if (keep) { MergedList.Add(s); } //...otherwise keep that element which is available across all lists in a merged list.
                }
            }
            return MergedList;
        }

        /// <summary>
        /// This method applies a pset or loads a signal using the method from the IParameterCollection interface.
        /// </summary>
        /// <param name="instrName"></param>
        /// <param name="pinName"></param>
        /// <param name="nspcpName"></param>
        /// <returns></returns>
        public Boolean ApplyOrLoad(String instrName, String pinName, String nspcpName, ref List<String> errors)
        {

            if (instrName == null || pinName == null || nspcpName == null) throw new ArgumentNullException("One or more arguments passed are null");
            ITCOBSTRs outErrorInfo;
            ITCOBSTRs outErrorNames;

            outErrorInfo = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            outErrorNames = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            IParameterCollection pColl = GetInstrumentInterface(instrName);
            using (new PsetAndSignalEditorFlowDomainUtil(m_frmPSetSignalsEditor.TMtoolID))
            {
            pColl.PSetSignalApply(nspcpName, pinName, ref outErrorNames, ref outErrorInfo);
            }
            //Prevent tdeHardwareStateChanged Event from being fired
            MyServices.CoreLanguageServiceCOM.DidExecute = false;

            if (!(outErrorInfo.Count > 0))
            {
                return true;
            }
            else
            {
                errors = CreateListfromBSTR((TERCOMMONOBJLib.ITCOBSTRs)outErrorInfo);
                return false;
            }
        }

        /// <summary>
        /// This method returns the possible choices for a given parameter.
        /// This method is called by the handler for the click event on a 
        /// parameter value cell of the parameter grid.
        /// </summary>
        /// <param name="pName"></param>
        /// <param name="pName">Logical Instrument</param>
        /// <returns></returns>
        public List<String> GetChoices(String pinName, String pName, String logicalInstrument)
        {

            if (pinName == null || pName == null || logicalInstrument == null) throw new ArgumentNullException("One of more arguments passed is null");
            String phyInstr;
            string[] strChoices;
            Array arrChoices;

            List<String> choiceList = new List<String>();

            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings choices = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            choices.Sort = false;

            IParameterCollection pColl = GetInstrumentInterface(logicalInstrument);


            try
            {
                phyInstr = GetPhysicalInstrument(pinName, logicalInstrument);
                pColl.ParameterChoices(pName, phyInstr, ref choices);
            }
            catch (Exception e)
            {
                throw new Exception("Error while getting choices for parameters " + e.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }
            //This is a technique to convert an array into type System.Array so that it can 
            //be passed as parameter to COM methods which use SAFEARRAY as their parameter types.
            strChoices = new string[0];
            arrChoices = strChoices as Array;

            arrChoices = choices.get_SA();

            foreach (String s in (String[])arrChoices)
            {
                choiceList.Add(s);
            }

            return (choiceList);
        }

        /// <summary>
        /// This method configures the parameter values/choices or whether they are 
        /// enabled/disabled in the parameter grid using the Configure method of the IParameterCollection interface.
        /// Names and Values are extracted from obj of type nspcpItem.
        /// Information for enabling/disabling columns is sent back to calling method through the enables.
        /// This method is called by the handler for the click event on a parameter value cell of the parameter grid.
        /// </summary>
        /// <param name="obj"></param>
        public Boolean[] Configure(ref NspcpItem obj)
        {
            if (obj == null) throw new ArgumentNullException("One of more arguments passed to Configure are null"); Boolean[] enables;

            Array arrEnables;
            int i = 1;
            ITCOBSTRs names;
            ITCOBSTRs values;

            enables = new Boolean[0];
            arrEnables = enables as Array;
            names = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            values = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());

            //We populate the name,value TCOBSTRs with the values from the object.
            //If this is the first time this method is being called, the object may
            //not have any information in it, so we bypass this step and receive 
            //all the information from the plugin.
            if (!(obj.NameValuePairs.Count == 0))
            {
                foreach (KeyValuePair<string, string> kvp in obj.NameValuePairs)
                {
                    names.Add(kvp.Key, i);
                    values.Add(kvp.Value, i);
                    i++;
                }
            }

            IParameterCollection pColl = GetInstrumentInterface(obj.propLogicalInstrument);

            //Update physical instrument information.
            obj.propPhysicalInstrument = GetPhysicalInstrument(obj.pinName, obj.propLogicalInstrument);

            try
            {
                //We now configure the plugin based on the name-value pairs passed. 
                //If configure is called the first time, its purpose is to mainly initialize everything
                //inside the plugin and get the parameter names.
                using (new PsetAndSignalEditorFlowDomainUtil(m_frmPSetSignalsEditor.TMtoolID))
                {
                pColl.Configure(ref names, ref values, obj.propPhysicalInstrument, obj.pinName, ref arrEnables);
            }
            }
            catch (Exception exp)
            {
                throw new Exception("Error while executing configure " + exp.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }

            //If this is the first time we call configure, them we populate the name/value pair.
            if (obj.NameValuePairs.Count == 0)
            {
                //Create Dictionary
                for (i = 1; i <= names.Count; i++)
                {
                    obj.NameValuePairs.Add(names[i], values[i]);
                }
            }
            else
            {
                for (i = 1; i <= names.Count; i++)
                {
                    obj.NameValuePairs[names[i]] = values[i];
                }
            }

            return (Boolean[])arrEnables;
        }

        /// <summary>
        /// This method checks if all the pins specified within the pingroup have the same definition for given PSet/Signal name. 
        /// It will basically compare the definition for any one pin against all the other pins on-by-one.
        /// This method is called when trying to view/edit existing PSet/Signal definitions for pin groups.
        /// </summary>
        /// <param name="pinSpec"></param>
        /// <param name="instrName"></param>
        /// <param name="nspcpName"></param>
        /// <returns></returns>
        public Boolean IsEqualDefinition(List<NspcpItem> objList)
        {
            if (objList == null) throw new ArgumentNullException("One or more arguments passed to IsEqualDefinition is null");

            if (objList.Count == 1) return true;        //If only one element, just return that

            NspcpItem startObj = objList[GlobalUtils.FIRST_ELEMENT];
            foreach (NspcpItem obj in objList)
            {
                if (!startObj.Compare(obj)) { return false; }
            }
            return true;
        }

        /// <summary>
        /// Returns the list of instruments associated with the
        /// provided list of pins. If the list of pins is empty then
        /// all the instruments are returned.
        /// </summary>
        /// <param name="PinList"></param>
        /// <returns></returns>
        public String[] GetInstruments(String PinList)
        {
            String tempString;

            //Check if pinlist
            String[] listPins = PinList.Split(',');

            //Get Common Instruments from pinlist.
            if (listPins.Length > 1)
            {
                List<List<String>> allLists = new List<List<string>>();

                //Go through all other pins to see if there is a common list.
                foreach (String pin in listPins)
                {
                   allLists.Add(new List<String>(GetInstruments(pin)));
                }

                List<String> commonElements = GetCommonElements(allLists);
                return (String[])commonElements.ToArray();
            }
            else
            {
                try
                {
                    if (m_editHelper == null) throw new NullReferenceException("Error in GetInstrument, CollectionEditHelper is NULL");
                    tempString = m_editHelper.get_InstrumentName(PinList);

                    if (tempString == null) throw new NullReferenceException("Received null value for the list of instruments");
                    String[] instrName = tempString.Split(',');

                    return instrName;
                }
                catch (Exception e)
                {
                    throw new Exception("Collection EditHelper failed while retrieving Instrument list " + e.Message);
                }
                finally
                {
                    //Prevent tdeHardwareStateChanged Event from being fired
                    MyServices.CoreLanguageServiceCOM.DidExecute = false;
                }
            }
        }
        /// <summary>
        /// Retuns the new button text and tool tip text
        /// </summary>
        /// <param name="obj">current object</param>
        /// <param name="buttonName">returns the button face string</param>
        /// <param name="title">returns the tooltip title</param>
        /// <param name="toolTip">returns the tooltip body</param>
        /// <param name="buttonEnabled">returns the button state</param>
        public void UpdateLoadSettingsButton(NspcpItem obj,out string buttonName,out string title,out string toolTip,out int buttonEnabled)
        {
            if (obj == null)
            {
                throw new ArgumentNullException("obj is null");
            }
            //get the IPlugin instance
            IPlugin pColl = GetInstrumentInterface(obj.propLogicalInstrument) as IPlugin;

            try
            {
                //if null fill default values and return
                if (pColl == null)
                {
                    buttonName = "Load Signal Settings";
                    title = null;
                    toolTip = null;
                    buttonEnabled = 1;
                }
                else
                {
                    //make call to plugin and get the new buttonface name
                    pColl.UpdateLoadSettings(obj.pinName, out buttonName, out title,out toolTip, out buttonEnabled);
                }
            }
            catch (Exception exp)
            {
                throw new Exception("Error while executing UpdateLoadSettings " + exp.Message);
            }
            finally
            {
                //Prevent tdeHardwareStateChanged Event from being fired
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }
        }


    }
}
