// ------------------------------------------------------------------
// (c) Teradyne, Inc, 2006
//     All Rights Reserved
// Inclusion of a copyright notice does not imply that this software
// has been published.  This software is the trade secret information
// of Teradyne, Inc.  Use of this software is only in accordance with
// the terms of a license agreement from Teradyne, Inc.
// ------------------------------------------------------------------
// ListViewColumnSorter.cs - Adaptation of http://support.microsoft.com/kb/319401
//   Impements sorting capability to list view columns. Supports both ascending and 
//   descending sort
// *************************************************************************************/
//   Revision history:
//   Author         Date        Description of change
//   =================================================================
//   shoban         9/19/2007   Created
// *************************************************************************************/

using System;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using System.Globalization;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    /// <summary>
    /// This class is an implementation of the 'IComparer' interface.
    /// </summary>
    class ListViewColumnSorter : IComparer
    {
        /// <summary>
        /// Specifies the column to be sorted
        /// </summary>
        private int m_columnToSort;
        /// <summary>
        /// Specifies the order in which to sort (i.e. 'Ascending').
        /// </summary>
        private SortOrder m_orderOfSort;
        /// <summary>
        /// Case insensitive comparer object
        /// </summary>
        private CaseInsensitiveComparer m_objectCompare;

        /// <summary>
        /// Class constructor.  Initializes various elements
        /// </summary>
        public ListViewColumnSorter()
        {
            // Initialize the column to '0'
            m_columnToSort = 0;

            // Initialize the sort order to 'none'
            m_orderOfSort = SortOrder.None;

            // Initialize the CaseInsensitiveComparer object
            m_objectCompare = new CaseInsensitiveComparer();
        }

        /// <summary>
        /// This method is inherited from the IComparer interface.  It compares the two objects passed using a case insensitive comparison.
        /// </summary>
        /// <param name="x">First object to be compared</param>
        /// <param name="y">Second object to be compared</param>
        /// <returns>The result of the comparison. "0" if equal, negative if 'x' is less than 'y' and positive if 'x' is greater than 'y'</returns>
        public int Compare(object x, object y)
        {
            int compareResult;
            ListViewItem listviewX, listviewY;

            // Cast the objects to be compared to ListViewItem objects
            listviewX = (ListViewItem)x;
            listviewY = (ListViewItem)y;

            if (listviewX == null || listviewY == null)
                throw new ArgumentException("Only list view items can be compared by ListViewColumnSorter");

            // Compare the two items
            compareResult = m_objectCompare.Compare(listviewX.SubItems[m_columnToSort].Text, listviewY.SubItems[m_columnToSort].Text);

            // Calculate correct return value based on object comparison
            if (m_orderOfSort == SortOrder.Ascending)
            {
                // Ascending sort is selected, return normal result of compare operation
                return compareResult;
            }
            else if (m_orderOfSort == SortOrder.Descending)
            {
                // Descending sort is selected, return negative result of compare operation
                return (-compareResult);
            }
            else
            {
                // Return '0' to indicate they are equal
                return 0;
            }
        }

        /// <summary>
        /// Gets or sets the number of the column to which to apply the sorting operation (Defaults to '0').
        /// </summary>
        public int SortColumn
        {
            set { m_columnToSort = value; }
            get { return m_columnToSort; }
        }

        /// <summary>
        /// Gets or sets the order of sorting to apply (for example, 'Ascending' or 'Descending').
        /// </summary>
        public SortOrder Order
        {
            set { m_orderOfSort = value; }
            get { return m_orderOfSort;  }
        }
   
    }
}
