
/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * PSetSheetEngine.cs file acts as an interface through which we can
 * perform various operations on PSet Sheet like Reading data from 
 * sheet, Updating definition of a Pset and getting the List of PSets.
 * ------------------------------------------------------------------
 */
/*
 ***************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =================================================================
 *   Grace Silvia   01/23/12    tersw00182305 - Added a condition check to UpdateSheet() to set formula to null, if there is only content value.
 *   Venkatachalam  09/27/11    Fixed tersw00180364 - Updated the Myservices.cs.
 *   Kalaivani C D  03/26/10    tersw00153690 - Fix for PSets sheet update from the editor when there is a Joblist sheet in the workbook.
 *   M. Raykin      10/30/07    Set "ScreenUpdateEnabled" property (IDTSheetSvcWorkbook) to false before updating DT data structures in UpdateSheet()
 *                              Reset it back to true at the end of the routine
 *   M. Raykin      10/24/07    Fixed tersw00113179
 *   Maria Raykin   10/19/07    Fixed bug in UpdateSheet() that caused an Assert in PSet sheet while deleting a row 
 *                              that has the same PSet name/different pin and if that row is creted from within Editor
 *   Maria Raykin   10/19/07    Fixed UpdateSheet() bug (updating existing PSet would write data beyond 
 *                              last parameter column).  No need to create new DMPsetspsetpinargs elements, but
 *                              update existing)
 *   Kiran Kumar    10/12/07    Fixed the defect : tersw00112494
 *   Kiran Kumar    09/26/07    Fixed the defect : tersw00111856
 *   Kiran Kumar    09/22/07    Fixed the defect : tersw00111764
 *   Kiran kumar    09/21/07    changed return of case insensitive to Case sensitive.
 *   Nadeem Khan    09/19/07    Replace FindMainSheetbyType with FindSheetbyType
 *                              Return Case Insensitive list of nspcp names.
 *   Kiran Kumar    09/18/07    Fixed the defect No: tersw00111445
 *   Kiran Kumar    08/16/07    Addressed Review comments
 *   Abdul Rajac    08/01/07    Created
 ***************************************************************************************
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Teradyne.Igxl.PSetSignalsEditor;
using Teradyne.Igxl.PSetSignalsEditor.PSetSignalsEditorServices;
using DMGRPSETSSHEETLIB;
using DTSHEETSERVICELib;
using CollectionEditHelperLib;
using DMGRJOBLISTSHEETLIB;
using DMGRLib;



namespace Teradyne.Igxl.PSetSignalsEditor
{
    public class PSetSheetEngine : ISheetEngine
    {
        bool is_disposed = false;
        private const int LIST_OF_ALL_PARAMS = 1;
        private const int FIRST_ARG_IN_SHEET = 2;
        private const int FIRST_PIN_NAME = 1;
        private const int FIRST_PSET = 1;
        private const int FIRST_PSET_NAME = 0;
        CCollectionEditHelper m_psetHelper;

        public PSetSheetEngine(ref CollectionEditHelperLib.CCollectionEditHelper pHelper)
        {
            try
            {
                m_psetHelper = pHelper;
            }
            catch (Exception e)
            {
                Console.Write("Exception caught in PSetSheetEngine {0}", e.StackTrace);
            }

        } 
       
        protected virtual void Dispose(bool disposing)
        {
            if (!is_disposed)
            {
                if (disposing)
                {
                    m_psetHelper = null;
                }
            }
            this.is_disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            //tell the garbage collector not to finalize
            GC.SuppressFinalize(this);
        }


        ~PSetSheetEngine()
        {
            Dispose(false);
        }

        #region SheetEngine Members

        /// <summary>
        /// The UpdateSheet() Method is used to update the PSet's definition in the PSet sheet.
        /// The parameter NspcpItem object contains the required data to update the sheet.
        /// While updating the sheet, if the sheet does not exist, this method will create one sheet 
        /// with the specified name and the definition of a pset will be updated in the sheet as new entry.
        /// The update sheet method will check, whether the pset name(which we are passing through 
        /// NspcpItem object) is matching with any of the psets names in the sheet. If it matches, 
        /// then it will check whether Pin name, which we are passing is matching with the name of the Pin 
        /// associated to the existing Pset in the sheet. If it matches then it will check for Instrument
        /// name matching. If it also matches, then the argument values of the existing Pset
        /// will be updated. If Instrument Name or Pin name are not matching, then the new entry for the
        /// existing PSet will be created. If PSet name is not matching, then new PSet will be
        /// created and the definition will be updated for the created PSet.
        /// </summary>
        /// <param name="obj">This parameter is of type NspcpItem. Which contains the information 
        /// of PSet Name, Pin Name, Instrument name, and Dictionary(Data Collection) of 
        /// Argument names and it's values as Key and Value pairs.</param>
        public void UpdateSheet(NspcpItem nspcpObj)
        {
            //Declare psetSheets to get the list of PSets sheets available
            IDMSheets psetSheets;

            DMPsetsSheet PSetSheet = null;
            DMPsetspsetpinargs argssInfo;
            DMPsetspsetpin pinInfo = null;
            DMPsetspset psetInfo;
            char[] delim = { ',' };
            String pSetNamesStr;
            String[] pSetNames;
            String pinNamesStr;
            String[] pinNames;
            Boolean isPinExist = false;
            int argColumn = FIRST_ARG_IN_SHEET;   // pset definitions start from the second parameter. 
            // The first parameter is a consolidation of all parameter names
            if ((nspcpObj == null) || (nspcpObj.propName == null) || (nspcpObj.pinName == null))
                throw new ArgumentNullException("obj or obj.propName or obj.propPinSpec are null");

            try
            {
                // following line will throw an exception if there is no PSet sheet available
                //PSetSheet = (DMPsetsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);
                psetSheets = (IDMSheets)MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);
                //psetSheet = (DMPsetsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);
                if ((psetSheets == null) || (psetSheets.Count == 0)) throw new ArgumentException();

                foreach (DMGRLib.IDMSheet item in psetSheets)
                {
                    DMPsetsSheet psetSheetTemp = item as DMPsetsSheet;
                    //if(&& psetSheetTemp.Psets.Has(nspcpObj.propName)
                    if (psetSheetTemp != null)
                    {
                        PSetSheet = psetSheetTemp;
                        break;
                    }
                }

                if (PSetSheet == null) throw new Exception("Invalid Pset sheet, the new PSets sheet created is not an active sheet");

            }
            catch (ArgumentException)
            {
                //Create New PSet Sheet with the default name.
                MyServices.DTSheetSvcCOM.CreateSheetFromDMGRType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET, null);
                try
                {
                    // following line will throw an exception if there is a JobList sheet in the workbook.
                    PSetSheet = (DMPsetsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);
                }
                catch (ArgumentException)
                {
                    DMJobList jobListSheet;
                    // get the job list sheet element.
                    jobListSheet = (DMJobList)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_JOBLISTSHEET);

                    IDTSheetSvcWorkbook wb = (IDTSheetSvcWorkbook)MyServices.DTSheetSvcCOM;
                    IExcelSheet psetsShtPtr;
                    // get the newly created PSets sheet pointer. 
                    wb.GetActiveSheet(out psetsShtPtr);

                    // Invalid Job List sheet.
                    if (jobListSheet.Jobs.Count == 0)
                    {
                        throw new Exception("Invalid JobList sheet, the new PSets sheet created is not an active sheet");
                    }                        

                    string activeJobNameStr;
                    // get the active job
                    activeJobNameStr = MyServices.DmgrServiceCOM.ContextMgr.JobState.ActiveJobName;                    

                    IDMJobListJob activeJobInfo;
                    // get the DM element of the active job.
                    activeJobInfo = jobListSheet.Jobs.Find(activeJobNameStr);

                    // add the newly created PSets sheet to the active Job
                    activeJobInfo.Psets = psetsShtPtr.name;
                    psetsShtPtr.Activate();

                    // get the DM element of the active PSets sheet to update the sheet parameters.
                    PSetSheet = (DMPsetsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);                  
                }
            }

            pSetNamesStr = PSetSheet.Psets.Names.CSL;
            pSetNames = pSetNamesStr.Split(delim);

            for (int parse = FIRST_PSET_NAME; parse < pSetNames.Length; parse++)
            {
                if (nspcpObj.propName.ToLower().Equals(pSetNames[parse].ToLower()))
                {
                    nspcpObj.propName = pSetNames[parse];
                    break;
                }
            }

            Boolean updateState = true;
            IDTSheetSvcWorkbook workbook = (IDTSheetSvcWorkbook)(MyServices.DTSheetSvcCOM);
            String key = "";
            // Set "ScreenUpdateEnabled" property to false in order to sheet reduce flickering
            try
            {
                if (workbook != null)
                {
                    updateState = workbook.ScreenUpdateEnabled;
                    workbook.ScreenUpdateEnabled = false;
                }

                if (PSetSheet.Psets.Has(nspcpObj.propName)) //We are updating an existing PSET or adding another pin (row)
                {
                    psetInfo = PSetSheet.Psets.Find(nspcpObj.propName);
                    pinNamesStr = psetInfo.Pins.Names.CSL; //the current list of pins the pset is defined for

                    String sheetPinList = String.Join(",", DecomposePinList(pinNamesStr)).ToLower(); ;
                    PARAMETERCOLLECTIONINTERFACELib.ITCOStrings sheetPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
                    sheetPins.set_TSL(",", sheetPinList);
                    pinNames = pinNamesStr.Split(delim); //the pin names selected in the editor.

                    //search if there is a row on the sheet with a pinspec that matches the pinsspec in the editor
                    for (int parse = FIRST_PSET_NAME; parse < pinNames.Length; parse++)
                    {
                        if (nspcpObj.pinName.ToLower().Equals(pinNames[parse].ToLower()))
                        {
                            nspcpObj.pinName = pinNames[parse]; //found. 
                            break;
                        }
                    }

                    String inputPinList = String.Join(",", DecomposePinList(nspcpObj.pinName)).ToLower(); ;
                    PARAMETERCOLLECTIONINTERFACELib.ITCOStrings inputPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
                    inputPins.set_TSL(",", inputPinList);
                    if (psetInfo.Pins.Has(nspcpObj.pinName))
                    {
                        isPinExist = true;
                        pinInfo = psetInfo.Pins.Find(nspcpObj.pinName);
                    }
                    else if (psetInfo.Pins.Has(inputPins.CSL))
                    {
                        isPinExist = true;
                        pinInfo = psetInfo.Pins.Find(inputPins.CSL);
                    }
                    else if (sheetPins.CSL.ToLower().Equals(inputPins.CSL.ToLower()))
                    {
                        isPinExist = true;
                        pinInfo = psetInfo.Pins.Find(pinNamesStr);
                    }

                    if (isPinExist)//PinSpec Found on Sheet. Just update the row
                    {
                        if (nspcpObj.propLogicalInstrument.ToLower().Equals(pinInfo.Insttype.ToLower()))
                        {
                            foreach (KeyValuePair<String, String> kvp in nspcpObj.NameValuePairs)
                            {
                                argssInfo = pinInfo.Argss[argColumn];
                                if (argssInfo != null)
                                {
                                    // Update values that have been changed only
                                    if (argssInfo.Name != kvp.Value)
                                    {
                                        object hasformula;
                                        hasformula = argssInfo.get_NameVF(DMGRPSETSSHEETLIB.tlPropertyType.tlPropertyFormula);
                                        if (hasformula != null)
                                        {
                                            argssInfo.set_NameVF(DMGRPSETSSHEETLIB.tlPropertyType.tlPropertyFormula, null);
                                        }
                                        argssInfo.Name = kvp.Value;
                                    }
                                }
                                argColumn++;
                            }
                        }
                        else
                        {
                            throw new ArgumentException("Instrument type does not match");
                        }
                    }
                    else
                    {
                        /* Create new pin information and attach to the existing PSet 
                       and add as a new row after the existing PSet's row.*/
                        pinInfo = psetInfo.Pins.CreateItem();
                        pinInfo.Name = nspcpObj.pinName;
                        pinInfo.Insttype = nspcpObj.propLogicalInstrument;
                        // pset's pin collection is 1-based; the object is inserted BEFORE this index
                        int PinIndex = psetInfo.Pins.Count;
                        psetInfo.Pins.Add(pinInfo, PinIndex);
                        argssInfo = pinInfo.Argss.CreateItem();
                        key.Trim();
                        foreach (String s in nspcpObj.NameValuePairs.Keys)
                        {
                            key = key + "," + s;
                        }
                        argssInfo.Name = key.TrimStart(',');

                        pinInfo.Argss.AddElement(argssInfo, LIST_OF_ALL_PARAMS);

                        foreach (KeyValuePair<String, String> kvp in nspcpObj.NameValuePairs)
                        {
                            argssInfo = pinInfo.Argss.CreateItem();
                            argssInfo.Name = kvp.Value;
                            pinInfo.Argss.Add(argssInfo, argColumn);
                            argColumn++;
                        }
                    }
                }
                else
                {
                    // Create New PSet And add at the end of the PSet sheet
                    DMPsetspset newPsetInfo;
                    
                    newPsetInfo = PSetSheet.Psets.CreateItem();
                    newPsetInfo.Name = nspcpObj.propName;

                    pinInfo = newPsetInfo.Pins.CreateItem();
                    pinInfo.Name = nspcpObj.pinName;
                    pinInfo.Insttype = nspcpObj.propLogicalInstrument;

                    newPsetInfo.Pins.Add(pinInfo, newPsetInfo.ElementIndex);
                    PSetSheet.Psets.Add(newPsetInfo, PSetSheet.Psets.Count + 1);

                    argssInfo = pinInfo.Argss.CreateItem();
                    key.Trim();
                    foreach (String s in nspcpObj.NameValuePairs.Keys)
                    {
                        key = key + "," + s;
                    }
                    argssInfo.Name = key.TrimStart(',');

                    pinInfo.Argss.AddElement(argssInfo, LIST_OF_ALL_PARAMS);

                    foreach (KeyValuePair<String, String> kvp in nspcpObj.NameValuePairs)
                    {
                        argssInfo = pinInfo.Argss.CreateItem();
                        argssInfo.Name = kvp.Value;
                        pinInfo.Argss.Add(argssInfo, argColumn);
                        argColumn++;
                    }
                }
            }
            finally
            {
                // restore sheet update state
                if (workbook != null)
                    workbook.ScreenUpdateEnabled = updateState;
            }
         }

        /// <summary>
        /// The ReadSheet() method is used to read the Pset's definition from Pset sheet 
        /// and populate the read data into an NspcpItem object and returns the same.
        /// It will perform the above functionality under the condition that, 
        /// if the Passed PsetName(NspcpName), pinName and instrumentName are matching with
        /// one of the existing entries in the Pset sheet.If The Pset sheet which we want
        /// to read from, does not exist then, it throws an InteropServices.COMException.
        /// if any of the Passed Parameters are not matching with any of the Sheet's Pset entries
        /// then this function returns null object. If Passed parameters contain null values, then
        /// The read function throws NullReferenceException.
        /// 
        /// </summary>
        /// <param name="pinName">pin name for which we want to read definition</param>
        /// <param name="instrName">Instrument name of which we want to read definition</param>
        /// <param name="nspcpName">Pset name of which we want to read definition from sheet</param>
        /// <returns>NspcpItem Object: This contains the Pset's definiton if the conditions 
        /// have been satisfied. Otherwise it contains null. </returns>
        public NspcpItem ReadSheet(String pinName, String instrName, String nspcpName)
        {
            //Declare psetSheets to get the list of PSets sheets available
            IDMSheets psetSheets;

            DMPsetsSheet psetSheet = null;
            DMPsetspset psetInfo;
            DMPsetspsetpin pinInfo = null;
            DMPsetspsetpinargs argssInfo;
            String[] paramNames; //to hold parameter names 
            String paramValue;
            String psetNamesStr;
            String pinNamesStr;
            String[] pSetNames;
            String[] pinNames;
            char[] delimiterChars = { ',' };


            if ((pinName == null) || (instrName == null) || (nspcpName == null))
                throw new ArgumentNullException(" pinName Or instrName Or PSet name are null");

            //Decompose the pin list that we got.
            String inputPinList = String.Join(",", DecomposePinList(pinName)).ToLower(); ;
            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings inputPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            inputPins.set_TSL(",", inputPinList);

            // The following line throws an COM Exception if the PSet Sheet is not available.
            //psetSheet = (DMPsetsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);

            psetSheets = (IDMSheets)MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);

            foreach (DMGRLib.IDMSheet item in psetSheets)
            {
                DMPsetsSheet psetSheetTemp = item as DMPsetsSheet;
                //if(&& psetSheetTemp.Psets.Has(nspcpName)
                if (psetSheetTemp != null )
                {
                    psetSheet = psetSheetTemp;
                    break;
                }
            }

            if (psetSheet == null) throw new SheetNotPresentException("There are no active PSet Sheets present in the workbook.");//Throw exception if sheet is null. 

            psetNamesStr = psetSheet.Psets.Names.CSL;
            pSetNames = psetNamesStr.Split(delimiterChars);
            //Case Insensitive search for NSPCP name.
            for (int parse = FIRST_PSET_NAME; parse < pSetNames.Length; parse++)
            {
                if (nspcpName.ToLower().Equals(pSetNames[parse].ToLower()))
                {
                    nspcpName = pSetNames[parse];
                    break;
                }
            }

            psetInfo = psetSheet.Psets.Find(nspcpName); // Throws a COM Exception if specified nspcp not found
            pinNamesStr = psetInfo.Pins.Names.CSL;

            String sheetPinNames = String.Join(",", DecomposePinList(pinNamesStr)).ToLower();
            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings readPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            readPins.set_TSL(",", sheetPinNames);

            pinNames = pinNamesStr.Split(delimiterChars);

            if (!pinNamesStr.ToLower().Equals(readPins.CSL.ToLower()))
            {
                pinInfo = psetInfo.Pins.Find(pinNamesStr);
            }
            else if (pinName.ToLower().Equals(readPins.CSL.ToLower()))
            {
                pinInfo = psetInfo.Pins.Find(pinName); // Throws an COM Exception if Specified Pin not found
            }
            else if (inputPins.CSL.ToLower().Equals(readPins.CSL.ToLower()))
            {
                pinInfo = psetInfo.Pins.Find(inputPins.CSL);
            }
            else
            {
                for (int parse = FIRST_PSET_NAME; parse < pinNames.Length; parse++)
                {
                    if (pinName.ToLower().Equals(pinNames[parse].ToLower()))
                    {
                        pinName = pinNames[parse];
                        pinInfo = psetInfo.Pins.Find(pinName);
                        break;
                    }
                }
            }
            if (pinInfo == null)
            {
                throw new ArgumentException("Invalid pin name Passed");
            }

            if ((nspcpName.ToLower().Equals(psetInfo.Name.ToLower())) &&
                ((pinName.ToLower().Equals(pinInfo.Name.ToLower())) ||
                (inputPins.CSL.ToLower().Equals(pinInfo.Name.ToLower())) ||
                (pinNamesStr.ToLower().Equals(pinInfo.Name.ToLower()))) &&
                (instrName.ToLower().Equals(pinInfo.Insttype.ToLower())))
            {
                argssInfo = (DMPsetspsetpinargs)pinInfo.Argss.get_Element(LIST_OF_ALL_PARAMS);
                paramValue = argssInfo.Name;
                paramNames = paramValue.Split(delimiterChars);
                Dictionary<String, String> KeyValuePairs = new Dictionary<String, String>();
                for (int pF = FIRST_ARG_IN_SHEET; pF <= paramNames.Length + 1; pF++)
                {
                    argssInfo = (DMPsetspsetpinargs)pinInfo.Argss.get_Element(pF);
                    paramValue = argssInfo.Name;
                    KeyValuePairs.Add(paramNames[pF - FIRST_ARG_IN_SHEET], paramValue);
                }
                return new NspcpItem(GlobalUtils.NspcpType.pset, nspcpName.ToLower(), pinName, pinInfo.Insttype,
                pinInfo.Insttype, KeyValuePairs.Count, KeyValuePairs);
            }
            else
            {
                throw new ArgumentException("Invalid Arguments Passed");
            }

        }
           
         


        /// <summary>
        /// The GetList() function is used to get the list of Psets from the Pset sheet
        /// which have been associated to the pinName and instrumentName which we are passing.
        /// </summary>
        /// <param name="pinName">Name of the pin for which we want to get the list of Psets that are 
        /// associated to, if the Pset is also associated to Instrument of Name instrName</param>
        /// <param name="instrName">Name of the instrument for which we want to get the list of Psets 
        /// that are associated to, if the Pset is also associated to the Pin of name pinName</param>
        /// <returns>An Array Of Sgnal names</returns>
        public String[] GetList(String pinName, String instrName)
        {
            DMPsetsSheet psetSheet;
            //Declare psetSheets to get the list of PSets sheets available
            IDMSheets psetSheets;
            String pinInfo;

            if ((pinName == null) || (instrName == null))
                throw new ArgumentNullException(" pinName or instrName are null");

            //Decompose the pin list that we got.
            String inputPinList = String.Join(",", DecomposePinList(pinName)).ToLower(); ;
            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings inputPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            inputPins.set_TSL(",", inputPinList);

            ArrayList psetList = new ArrayList();
            try
            {
                // Following throws an exception if pset Sheet is not available
                //Get the list of Pset sheets by using FindSheetsByType
            psetSheets = (IDMSheets)MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);           
            if ((psetSheets == null)||(psetSheets.Count == 0)) throw new ArgumentException();
                //Loop through each Pset sheet
            foreach (DMGRLib.IDMSheet sheet in psetSheets)
            {
                    psetSheet = (DMPsetsSheet)sheet;
                    for (int pSetId = FIRST_PSET; pSetId <= psetSheet.Psets.Count; pSetId++)
                    {
                        for (int pinId = FIRST_PIN_NAME; pinId <= psetSheet.Psets[pSetId].Pins.Count; pinId++)
                        {
                            //make sure the instruments match
                            if (instrName.ToLower().Equals(psetSheet.Psets[pSetId].Pins[pinId].Insttype.ToLower()))
                            {
                                //if(pinName.ToLower().Equals(psetSheet.Psets[pSetId].Pins.Names.CSL.ToLower()))
                                ////now check the pins match
                                pinInfo = psetSheet.Psets[pSetId].Pins[pinId].Name; // Throws COM exception if invalid pin is specified.
                                ////Decompose this Row's pin list
                                String thisRowPinList = String.Join(",", DecomposePinList(pinInfo)).ToLower(); ;
                                PARAMETERCOLLECTIONINTERFACELib.ITCOStrings thisRowPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
                                thisRowPins.set_TSL(",", thisRowPinList);

                                //if(inputPins.CSL == thisRowPins.CSL)
                                if (inputPins.Count == thisRowPins.Count)//if its not the same number of pins than it does not match
                                {
                                    Boolean match = true;
                                    //iterate through the input pins and make sure they all exist in this row
                                    for (int pinIndex = 1; pinIndex <= inputPins.Count; pinIndex++)
                                    {
                                        if (!thisRowPins.Has(inputPins[pinIndex]))
                                        {
                                            match = false;
                                            break;
                                        }
                                    }
                                    if (match)
                                        psetList.Add(psetSheet.Psets[pSetId].Name.ToLower());
                                }
                            }
                        }
                    }
                }
            }
            catch (System.ArgumentException)
            {
                throw new SheetNotPresentException("There are no active PSet Sheets present in the workbook.");
            }

            return (String[])psetList.ToArray(typeof(String));            
        }
        /// <summary>
        /// Decomposes a Pin Spec into a list of pin names that compose that pinspec.
        /// </summary>
        /// <param name="pinspec">PinSpec Name</param>
        /// <returns>List of pins</returns>
        public String[] DecomposePinList(String pinspec)
        {

            if (pinspec == null) throw new ArgumentNullException("One of more arguments passed to DecomposePinList is null");
            Array arrPinList;
            String[] strPinList;
            int pinCount;

            //TypeCast Array to String[] so that it can be passed as a parameter
            //to the DecomposePinList method.
            strPinList = new String[0];
            arrPinList = strPinList as Array;

            try
            {
                //Decompose the pin spec to get the list of pins.
                
                if(m_psetHelper.IsValidPinSpec(pinspec))
                    MyServices.DmgrServiceCOM.DecomposePinList(pinspec, out arrPinList, out pinCount);
            }
            catch (Exception e)
            {
                throw new ServicesException("Error in DMGR Service while Decomposing Pin List" + e.Message);
            }

            return ((String[])arrPinList);
        }
        #endregion

    }
}
