/* Copyright (C) 2007 Teradyne, Inc. All rights reserved.
 * -------------------------------------------------------------------
 *  This document contains proprietary and confidential information of Teradyne,
 * Inc. and is tendered subject to the condition that the information (a) be
 * retained in confidence and (b) not be used or incorporated in any product
 * except with the express written consent of Teradyne, Inc.
 *
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * UserInterfaceEngine.cs file acts as an interface through which we can
 * perform various operations on Compare window and Main window like 
 * populating the definition of nspcps to the grid view controls, and
 * and comparing two existing nspcps etc..
 * ------------------------------------------------------------------
 */

/*
 ***************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =================================================================
 *   Kannan K        06/08/11    Added multi sheet support to WaveDef Sheet.
 *   Gokuldass       04/28/11    tersw00174595 - Modified PopulateColumn() to update the tooltip in the appropriate invalid cell value and
 *                               modified ClearCell() to clear the error only if the cell is selected.
 *   Bruce Brolsma   04/19/11    Fixed tersw00175087 - Modified ReportErrorInListBox() to initialize the error grid if needed
 *   Rajasekaran     16/03/11    Fixed tersw00173759 - Modified ReportErrorInCell(),MarkErrorCell()and added method EnlargeErrorPane() to enlarge the error pane if only errors occurs
 *   Matt Dalton     09/16/10    Fixed tersw00158817 - Made string comparison case-insensitive when looking for a value in a drop-down menu.
 *                               Also fixed some coding standard violations.
 *   Subeetha        10/23/09    Fixed tersw00151138 - Resolved multiple interactive issue by overridding DataGridViewComboboxCell features.
 *   John Devadas    07/23/09    Fixed tersw00145698 - in ClearAll() function - clears the grid if grid is errors i.e Pink color
 *   Shoban Jayaraj  03/07/08    Fixed tersw00118428 - invalid choices in dropdown are cleared
 *   V.Mariappa Raja 11/28/07    Fixed tersw00114322.
 *   Suresh VA       11/26/07    Fixed tersw00114460.
 *   Shoban Jayaraj  10/23/07    Fix for tersw00113059 - change text color to blue for text 
 *                               elements in grid
 *   Andy Kittross   10/15/07    Remove Add/Remove/Left/Right compare functionality.  Now there
 *                               is just the multi-column grid.
 *   Kiran           09/26/07    Fixed the defect : tersw00111864
 *   Kiran           09/21/07    Fixed Defect Id : tersw00111481 
 *   Kiran           09/12/07    Created Compare window functionalities
 *   Nadeem Khan     08/31/07    Created
 ***************************************************************************************
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using Teradyne.Igxl.PSetSignalsEditor.PSetSignalsEditorServices;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    /// <summary>
    /// The User Interface engine provides methods to configure and 
    /// populate the parameter grid and the error component.
    /// </summary>
    public class UserInterfaceEngine
    {
        NspcpItemsGridView m_paramGrid;           //Reference to the Parameter Grid
        DataGridView m_errorGrid;           //Reference to the Error Grid.
        InstrumentEngine m_InstrEngine;     // Reference to the Instrument engine      
        SplitContainer m_split;
        int m_GridHeight;
        private Boolean m_GridInitialized = false;  // Flag to say whether Grid has been initialized with values
        public Boolean IsInitialized
        {
            get { return m_GridInitialized; }
            set { m_GridInitialized = value; }
        }

        public UserInterfaceEngine(ref NspcpItemsGridView pGrid,ref DataGridView eGrid, ref InstrumentEngine ie)
        {
            m_GridInitialized = false;
            m_paramGrid = pGrid;
            m_errorGrid = eGrid;
            m_InstrEngine = ie;
            m_split = null;
            //Attach a method to handle errors occurring on the data grid.
            m_paramGrid.DataError += new DataGridViewDataErrorEventHandler(m_paramGrid_DataError);
        }

      

        #region Error Handling for Data Grid

        /// <summary>
        /// Handle the errors occuring on the data grid.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void m_paramGrid_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            MessageBox.Show("Error in Data Grid" + e.Context.ToString());
        }

        #endregion

        #region Parameter Grid Related Operations

        /// <summary>
       /// This method reads the parameter grid 
       /// and updates the nspcp objects of the currently displayed nspcps with 
       /// their corresponding values in the grid.
       /// </summary>
       /// <param name="objList"></param>
        public void CollectGridInformation(ref Dictionary<String,NspcpItem> objTable) 
        {
            //Get the number of rows in the grid
            int numRows = m_paramGrid.RowCount;

            if (objTable != null)
            {
                foreach (String key in objTable.Keys)
                {
                    NspcpItem tmpObj = objTable[key];
                    CollectGridInformation(ref tmpObj);
                    ////Scan through all rows 
                    //for (int i = 0; i < numRows; i++)
                    //{
                    //    //Get the parameter name for that row
                    //    String paramName = (String)m_paramGrid[0, i].Value;
                    //    //Update the nspcp object dictionary with the grid valud.
                    //    kvp.Value.NameValuePairs[paramName] = (String)m_paramGrid[kvp.Key, i].Value;
                    //}
                }
            }
        }

        public void CollectGridInformation(ref NspcpItem obj)
        {
            String paramValue;
            String paramName;
            //Get the number of rows in the grid
            int numRows = m_paramGrid.RowCount;

            //Scan through all rows 
            for (int i = 0; i < numRows; i++)
            {
                //Get the parameter name for that row
                paramName = (String)m_paramGrid[0, i].Value;
                paramValue = (String)m_paramGrid[obj.propName, i].Value;
                //We do not allow null values, we replace them with blanks instead
                if (paramValue == null)
                {
                    paramValue = String.Empty;
                }
                //Update the nspcp object dictionary with the grid value.
                obj.NameValuePairs[paramName] = paramValue;
            }           
        }

        /// <summary>
        /// Initialize the parameter grid by adding the list of rows
        /// for the 'Parameter Name' column. We also make this column frozen
        /// </summary>
        /// <param name="names"></param>
        public void InitDisplay(Dictionary<String,String>.KeyCollection names)
        {
            int i = 0;
            //Add the requisite number of rows to the grid
            m_paramGrid.Rows.Add(names.Count);
            foreach (String n in names)
            {
                m_paramGrid[0, i].Value  = n;
                i++;
            }
            m_paramGrid.Columns[0].Frozen = true;
        }

        /// <summary>
        /// Create a column in the grid for the new nspcp.
        /// </summary>
        /// <param name="obj"></param>
        public void AddColumn(NspcpItem obj)
        {
            if (m_paramGrid.ColumnCount < 1)
            {
                throw new PSetSignalsEditor.Exceptions.DisplayUnInitializedException("Paramater Grid needs to be initialized with parameter names before this method is called.");
            }

            //Add only if the column does not already exist
            if (!m_paramGrid.Columns.Contains(obj.propName))
            {
                ++m_paramGrid.ColumnCount;              //Add a new column to the grid
                m_paramGrid.Columns[m_paramGrid.Columns.Count - 1].Name = obj.propName;     //Set the column header
                m_paramGrid.Columns[m_paramGrid.Columns.Count - 1].SortMode = DataGridViewColumnSortMode.NotSortable;
                m_paramGrid.Columns[m_paramGrid.Columns.Count - 1].Frozen = false;
            }
        }

        /// <summary>
        /// This method adds a new column to the display with the required 
        /// parameter values. However, we have to make sure if the display is 
        /// initialized with the parameter names before we start adding more columns
        /// </summary>
        /// <param name="obj"></param>
        public void PopulateColumn(NspcpItem obj)
        {
            try
            {
                int i = 0;
                //Populate with nspcp definition.
                foreach (String value in obj.NameValuePairs.Values)
                {
                    //Set cell dropdowns
                    List<String> choices;
                    choices = new List<string>();
                    Color oldColor = m_paramGrid[obj.propName, i].Style.ForeColor;
                    Color oldbackColor = m_paramGrid[obj.propName, i].Style.BackColor;

                    //Set the current cell to DataGridViewTextBoxCell object by default
                    //later it can be modified to point to an object of type DataGridViewComboBoxCell,
                    //depending upon whether the cell will host a set number of choices.
                    try
                    {
                        m_paramGrid[obj.propName, i] = new DataGridViewTextBoxCell();
                    }
                    catch (InvalidOperationException)
                    {
                        // do nothing This exception will be thrown when Row Index and column index are equal to 2
                    }
                    PopulateCellDropDown(m_paramGrid[obj.propName, i].ColumnIndex, i, obj.propLogicalInstrument, obj.pinName, ref choices, value);

                    //Make sure the cell is not selected, otherwise the grid 
                    //locks the cell object and does not allow value updates on it.
                    if (m_paramGrid[obj.propName, i].Selected)
                    {
                        m_paramGrid[obj.propName, i].Selected = false;
                    }

                    if (choices.Count > 0) //Check if its a dropdown cell.
                    {
                        if (!value.Equals(String.Empty)) //Check if value is null
                        {
                            DataGridViewCell cell = m_paramGrid[obj.propName, i];

                            cell.Style.ForeColor = oldColor;    //Restore the color.
                            cell.Style.BackColor = oldbackColor;    //Restore the back color.

                            // Look for a matching entry in the choices list.
                            // We can't just use the "Contains" method from the
                            // list because we want to do case-insensitive 
                            // comparisons.  So we'll loop through the list
                            // comparing each one individually.
                            string foundMatch = String.Empty;
                            foreach(string choice in choices)
                            {
                                if(string.Compare(value, choice, true) == 0)
                                {
                                    // Note:  Even though these strings match,
                                    // they may have different capitalizations
                                    // (e.g. TRUE vs. True).  We want to display
                                    // the one from the choices list, not 
                                    // the value.  That's the capitalization scheme
                                    // that the plug-in is expecting.
                                    foundMatch = choice;
                                    break;
                                }
                            }

                            if (foundMatch == String.Empty)  //If the value set is not part of the dropdown
                            {                 //Report and error on that cell.
                                cell.Value = String.Empty; // reset to that we show the empty combo box

                                String paramName = (String)m_paramGrid[0, i].Value;
                                ReportErrorInCell(obj.propName, paramName, "Older value has been cleared as it is not a valid option. Please reselect from dropdown list.");
                            }
                            else
                            {
                                cell.Value = foundMatch; //Set the cell value.
                            }


                        }
                    }
                    else
                    {
                        DataGridViewCell cell = m_paramGrid[obj.propName, i];
                        cell.Value = value;
                        //Find the appropriate tooltip error from collection and update the cell.
                        String errCellValue;
                        for (int idx = 0; idx < obj.propErrName.Count; idx++)
                        {
                            obj.NameValuePairs.TryGetValue(obj.propErrName[idx], out errCellValue);
                            if (value.Equals(errCellValue))
                                cell.ToolTipText = obj.propErrInfo[idx];
                        }
                        cell.Style.ForeColor = oldColor;    //Restore the color.
                        cell.Style.BackColor = oldbackColor;    //Restore the back color.
                    }
                    i++;
                }
            }
            catch (Exception exp)
            {
                throw new Exception(exp.Message);
            }
        }

        /// <summary>
        /// This method is like PopulateColumn except it does not
        /// provide dropdown choice values and marks all the cells
        /// disabled.
        /// </summary>
        public void PopulateColumnWithoutDropdowns(NspcpItem obj)
        {
            try
            {
                int i = 0;
                //Populate with nspcp definition.
                foreach (String value in obj.NameValuePairs.Values)
                {
                    //Make sure the cell is not selected, otherwise the grid 
                    //locks the cell object and does not allow value updates on it.
                    if (m_paramGrid[obj.propName, i].Selected)
                    {
                        m_paramGrid[obj.propName, i].Selected = false;
                    }

                    m_paramGrid[obj.propName, i].Value = value; //Set the value.
                    m_paramGrid[obj.propName, i].Style.BackColor = SystemColors.Control; //Disable the cell.
                    i++;
                }
            }
            catch (Exception exp)
            {
                throw new Exception(exp.Message);
            }
        }

        private List<String> GetWaveDefs()
        {
            List<String> waveDefs = new List<string>();

            DMGRLib.IDMSheets sheets = MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET) as DMGRLib.IDMSheets;

            if(sheets == null || sheets.Count == 0) throw new Exception("No WaveDef Sheet Defined");

            foreach (DMGRLib.IDMSheet item in sheets)
            {
                DMGRWAVEDEFINITIONSHEETLIB.DMWavedefinitionSheet DMWaveDefSheet = item as DMGRWAVEDEFINITIONSHEETLIB.DMWavedefinitionSheet;
            if (DMWaveDefSheet == null)
            {
                throw new Exception("No WaveDef Sheet Defined");
            }
            //Get the WaveDef from the WaveDefinition Sheet
            foreach (DMGRWAVEDEFINITIONSHEETLIB.DMWaveDefinition DMWaveDef in DMWaveDefSheet.WaveDefinitions)
            {
                waveDefs.Add(DMWaveDef.Name);
            }
            }

            CORELANGUAGELib.ExecWaveDefinitions execWaveDefs = MyServices.CoreLanguageServiceCOM.Exec.WaveDefinitions;           

            return waveDefs;
        }

        public void DeleteColumn(NspcpItem obj)
        {
            m_paramGrid.Columns.Remove(obj.propName);
        }

        /// <summary>
        /// This method sets the dropdown choices for a parameter 
        /// value cell given the set of choices in String array.
        /// </summary>
        /// <param name="pName"></param>
        /// <param name="choices"></param>
        public void PopulateCellDropDown(int colIndex, int rowIndex, String logicalInstrument, String pinName, ref List<String> choices, String value)
        {
            String pName;
            //Get the parameter name
            pName = (String)m_paramGrid[0, rowIndex].Value;

            //Handle the WaveDef parameter case as an exception 
            //since we get choices from WaveDef Sheet.
            if (pName.Equals(GlobalUtils.WAVEDEF_PARAM_NAME))
            {
                try
                {
                    choices = GetWaveDefs();
                }
                catch (Exception) 
                {
                    //Do nothing, we simple do not fill choices (Hence dropdown is not populated).
                }
            }
            else //Get choices for all other parameters from the instrument engine.
            {
                choices = m_InstrEngine.GetChoices(pinName, pName, logicalInstrument);
            }

            //Populate dropdown.
            if (choices.Count > 0)
            {
                //We are required to create a new instance of the cell box for each column.
                DataGridViewComboBoxCell cmbCell = new DataGridViewComboBoxCellEx();

                foreach (String s in choices)
                {
                    cmbCell.Items.Add(s);
                }
                try
                {
                    m_paramGrid[colIndex, rowIndex] = cmbCell;
                }
                catch (InvalidOperationException)
                {
                    // do Nothing, This Exception will be thrown only when Row Index and Col Index are equal
                }
            }
        }


        /// <summary>
        /// This method enables/disables cells within the parameter grid given the boolean 
        /// settings inside the enableArray. This is called once the information about which 
        /// parameter values cells should be left enabled/disabled is received from the 
        /// instrument plug-ins (using the configure method).
        /// </summary>
        /// <param name="enables"></param>
        public void ConfigureCells(Boolean[] enables, String nspcpName)
        {
            try
            {
                if (enables.Length != m_paramGrid.Rows.Count)
                {
                    throw new Exception("List of enables is not same as the number of rows");
                }

                for (int i = 0; i < m_paramGrid.Rows.Count; i++)
                {
                    DataGridViewCell cell = m_paramGrid[nspcpName, i];
                    ConfigureCell(ref cell, (Boolean)enables.GetValue(i));
                }
            }
            catch (Exception exp)
            {
                throw new Exception(exp.Message);
            }
        }

        private void ConfigureCell(ref DataGridViewCell Cell,Boolean state)
        {
            if (!state)
            {
                if (Cell is DataGridViewComboBoxCell)
                {
                    (Cell as DataGridViewComboBoxCell).Style.BackColor = SystemColors.Control;
                }
                else
                {
                    Cell.Style.BackColor = SystemColors.Control;
                }
                Cell.ReadOnly = true;
                Cell.Style.ForeColor = Color.Gray;
            }
            else
            {
                if (Cell is DataGridViewComboBoxCell)
                {
                    if ((Cell as DataGridViewComboBoxCell).Style.BackColor == SystemColors.Control)
                    {
                        (Cell as DataGridViewComboBoxCell).Style.BackColor = SystemColors.Window;
                    }
                }
                else
                {
                    if (Cell.Style.BackColor == SystemColors.Control) 
                    { 
                        Cell.Style.BackColor = SystemColors.Window; 
                    }
                }

                Cell.ReadOnly = false;
                if (Cell.Style.ForeColor == Color.Gray) 
                { 
                    Cell.Style.ForeColor = Color.Black; 
                }
            }
        }

        /// <summary>
        /// Compare the object passed with the definition displayed and mark all differences.
        /// </summary>
        /// <param name="obj"></param>
        public void CompareAndMark(NspcpItem obj)
        {
            if (obj == null)
            {
                throw new ArgumentNullException("One or more arguments passed is null");
            }

            //Check if the NSPCP item is actually displayed on the display
            if (m_paramGrid.Columns.Contains(obj.propName))
            {
                //Number of rows displayed in the grid.
                int numRows = m_paramGrid.Rows.Count;
                if (numRows != obj.NameValuePairs.Count)
                {
                    throw new Exception("The number of parameters of the compared object do not match the number of parameters displayed on the grid");
                }
                
                for (int row = 0; row < numRows; row++)
                {
                    String paramName = (String)m_paramGrid[0, row].Value;
                    //Compare parameter value in grid with parameter value in object.
                    String GridValue = (String)m_paramGrid[obj.propName, row].Value;
                    String ObjValue = obj.NameValuePairs[paramName];
                    if (GridValue != null) //This could be the case for an un-initialized dropdown.
                    {
                        if (!GridValue.Equals(ObjValue))
                        {
                            //If different, then paint the difference.
                            m_paramGrid[obj.propName, row].Style.ForeColor = GlobalUtils.CELL_DIFF_COLOR;
                        }
                    }
                }
            }
            else
            {
                throw new Exception("The item to compare does not exist on the grid");
            }
        }
        /// <summary>
        /// Identifies the first error in the grid, acoordingly parameter grid 
        /// scroll bars will point to the first error in the grid
        /// </summary>
        public void ScrollToFirstErrorInGrid()
        {
            int nrows = m_paramGrid.Rows.Count;
            int ncols = m_paramGrid.Columns.Count;

            for (int col = 1; col < ncols; col++)
            {
                for (int row = 1; row < nrows; row++)
                {
                    DataGridViewCell cell = m_paramGrid[col, row];
                    if (cell.Style.BackColor == ERROR_CELL_BACKGROUND)
                    {
                        m_paramGrid.FirstDisplayedScrollingRowIndex = row;
                        m_paramGrid.FirstDisplayedScrollingColumnIndex = col;
                        return;
                    }
                }
            }
        }

        #endregion


        #region Methods Related to Error Reporting
       
        private readonly Color ERROR_CELL_BACKGROUND = Color.Pink;
        private readonly Color DEFAULT_CELL_BACKGROUND = Color.White;
        private const int ERROR_GRID_NSPCPNAME_COLUMN = 0;
        private const int ERROR_GRID_MESSAGE_COLUMN = 1;
        private const String GENERAL_ERROR = "General";
        private const String NSPCP_PIN_FORMAT = "{0}:{1}";

        /// <summary>
        /// This method reports an error message to the cell containing the value of 
        /// the parameter in parameterName. This basically highlights the value cell for 
        /// that parameter for a selected signal / pset in a different color and sets 
        /// the tooltip of that cell to contain the error message.
        /// </summary>
        /// <param name="nspcpName">The name of the nspcpitem whose parameter has an error</param>
        /// <param name="pName">The parameter name that caused the error</param>
        /// <param name="errInfo">The actual error message</param>
        public void ReportErrorInCell(String nspcpName, String pName, String errInfo)
        {
            if (nspcpName == null || pName == null || errInfo == null)
            {
                throw new ArgumentNullException("nspcpName or pName or errInfo for UserInterfaceEngine.ReportErrorInCell is null");
            }

            if (!m_paramGrid.Columns.Contains(nspcpName))
            {
                // We've got an unknown signal. Pass on to error grid instead
                List<String> errors = new List<String>();
                errors.Add(errInfo);
                //if the error string added,check if error grid initialized or not
                //if not initialized then initialize the error pane
                if (m_errorGrid.Rows.Count == 0 && m_split != null)
                    EnlargeErrorPane();
                ReportErrorInListBox(nspcpName, errors);
                return;
            }

            int col = m_paramGrid.Columns[nspcpName].Index;

            MarkErrorCell(nspcpName, pName, errInfo, col);
        }

        /// <summary>
        /// Report errors (if any) that may have occurred during
        /// validation.
        /// </summary>
        /// <param name="objList"></param>
        public void ReportErrors(ref Dictionary<String, NspcpItem> objList)
        {
            int idx;
            String errInfo;
            //Iterate through entire nspcp collection to report errors
            foreach (NspcpItem obj in objList.Values)
            {
                if (obj.propErrInfo.Count > 0)
                {
                    idx = 0;
                    //Report the list of all errors associated with that nspcp.
                    foreach (String errName in obj.propErrName)
                    {
                        errInfo = obj.propErrInfo[idx];
                        ReportErrorInCell(obj.propName, errName, errInfo);
                        idx++;
                    }
                }
            }
        }
        /// <summary>
        /// Used to initialize the splitcontainer and errorpane height.
        /// </summary>
        public void initializeContainer(ref SplitContainer split, int height)
        {
            m_split = split;
            m_GridHeight = height - 30;
        }
        /// <summary>
        /// Used to Enlarge the Error Grid when it has the error Information.
        /// </summary>
        private void EnlargeErrorPane()
        {
            m_errorGrid.Height = m_GridHeight;
            //Splitter Distance
            m_split.SplitterDistance = 472;
            //Initialize the Error Grid
            m_errorGrid.ColumnCount = 2;
            m_errorGrid.BackgroundColor = Control.DefaultBackColor; // backcolor of interior area outside cells
            m_errorGrid.ColumnHeadersDefaultCellStyle.Font = new Font(m_errorGrid.Font, FontStyle.Bold);
            m_errorGrid.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.DisplayedCellsExceptHeaders;
            m_errorGrid.ColumnHeadersBorderStyle = DataGridViewHeaderBorderStyle.Single;
            m_errorGrid.CellBorderStyle = DataGridViewCellBorderStyle.Single;
            m_errorGrid.RowHeadersVisible = false;
            m_errorGrid.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells;
            m_errorGrid.AutoSizeColumnsMode = System.Windows.Forms.DataGridViewAutoSizeColumnsMode.Fill; // important when resizing                       
            m_errorGrid.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right | System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Bottom)));
            m_errorGrid.Columns[0].Name = "Type";
            m_errorGrid.Columns[0].Width = 300;
            m_errorGrid.Columns[0].MinimumWidth = 150;
            m_errorGrid.Columns[1].Name = "Error";
            m_errorGrid.Columns[1].Width = 400;
            m_errorGrid.Columns[0].ReadOnly = true;
            m_errorGrid.Columns[1].ReadOnly = true;
            m_errorGrid.Columns[0].SortMode = DataGridViewColumnSortMode.NotSortable;
            m_errorGrid.Columns[1].SortMode = DataGridViewColumnSortMode.NotSortable;
            m_errorGrid.MultiSelect = true;
            m_errorGrid.SelectionMode = DataGridViewSelectionMode.CellSelect;
            // Enable single-click-to-edit for editable cells:
            m_errorGrid.EditMode = System.Windows.Forms.DataGridViewEditMode.EditOnEnter;
            // Suppress empty last row:
            m_errorGrid.AllowUserToAddRows = false;
        }

        /// <summary>
        /// Given a parameter name and the column index, marks the cell in ERROR_CELL_BACKGROUND 
        /// and sets the error text as a tool-tip
        /// </summary>
        /// 
        /// Note: No validation on column index and pName are done. The caller should validate.
        /// 
        /// <param name="nspcpName">The nspcp name</param>
        /// <param name="pName">The parameter name that caused the error</param>
        /// <param name="errInfo">The actual error message</param>
        /// <param name="col">The column index of the parameter to mark</param>
        private void MarkErrorCell(String nspcpName, String pName, String errInfo, int col)
        {
            int rowCount = m_paramGrid.Rows.Count;

            for (int row = 0; row < rowCount; row++)
            {
                DataGridViewRow dgRow = m_paramGrid.Rows[row];
                String row_name = (String)dgRow.Cells[0].Value;
                if (row_name.Equals(pName))
                {
                    dgRow.Cells[col].ToolTipText = errInfo;
                    dgRow.Cells[col].Style.BackColor = ERROR_CELL_BACKGROUND;
                    return;
                }
            }
            // We've got an unknown pin. Pass on to error grid instead
            List<String> errors = new List<String>();
            //if the error string added,check if error grid initialized or not
            //if not initialized then initialize the error pane
            if (m_errorGrid.Rows.Count == 0 && m_split != null)
                EnlargeErrorPane();
            errors.Add(errInfo);
            ReportErrorInListBox(String.Format(NSPCP_PIN_FORMAT, nspcpName, pName), errors);

        }

        /// <summary>
        /// This method will report all the errors in the error list box which cannot 
        /// be reported on the parameter value cells.
        /// </summary>
        /// 
        /// <param name="nspcpName">The name of the nspcp to tag the errror to. 
        /// Note: This can be null, in which case, they appear as general errors</param>
        /// <param name="errInfo">The general error message</param>
        public void ReportErrorInListBox(String nspcpName, List<String> errInfo)
        {
            if (errInfo == null)
            {
                throw new ArgumentNullException("errInfo cannot be null");
            }

            if (nspcpName == null)
            {
                nspcpName = GENERAL_ERROR;
            }

            int row;
            foreach (String err in errInfo)
            {
                // tersw00175087   Initialize the error grid if needed
                if (m_errorGrid.Rows.Count == 0 && m_split != null)
                {
                    EnlargeErrorPane();
                }
                row = m_errorGrid.Rows.Add();
                m_errorGrid.Rows[row].Cells[ERROR_GRID_NSPCPNAME_COLUMN].Value = nspcpName;
                m_errorGrid.Rows[row].Cells[ERROR_GRID_MESSAGE_COLUMN].Value = err;
            }

            ///Added the below code Defect tersw00114460           
            m_errorGrid.Columns[ERROR_GRID_MESSAGE_COLUMN].ReadOnly = false;
        }


        /// <summary>
        /// Clears the error grid and the parameter grid off all errors
        /// </summary>
        public void ClearAll()
        {
            // Clear the errror grid
            int count = m_errorGrid.Rows.Count;

            while (count > 0)
            {
                m_errorGrid.Rows.RemoveAt(0);
                count--;
            }

            // clear the parameter grid of all tool-tips and reset background color
            int rows = m_paramGrid.Rows.Count;
            int cols = m_paramGrid.Columns.Count;

            for (int r = 0; r < rows; r++)
            {
                for (int c = 1; c < cols; c++)
                {
                    DataGridViewCell cell = m_paramGrid.Rows[r].Cells[c];
                    if (cell.ToolTipText.Length != 0)
                    {
                        cell.ToolTipText = String.Empty;
                        cell.Style.BackColor = DEFAULT_CELL_BACKGROUND;
                    }
                    else if (cell.Style.BackColor == ERROR_CELL_BACKGROUND)
                    {
                        cell.Style.BackColor = DEFAULT_CELL_BACKGROUND;
                    }
                }
            }
        }

        /// <summary>
        /// This method clears the font color of all the text that has been modified 
        /// and turned to blue.
        /// </summary>
        public void RestoreTextColor()
        {
            int nrows = m_paramGrid.Rows.Count;
            int ncols = m_paramGrid.Columns.Count;
            for (int row = 0; row < nrows; row++)
            {
                for (int col = 1; col < ncols; col++)
                {
                    DataGridViewCell cell = m_paramGrid[col, row];
                    if (cell.Style.ForeColor == GlobalUtils.CELL_CHANGE_COLOR)
                    {
                        cell.Style.ForeColor = GlobalUtils.CELL_STANDARD_COLOR;
                    }
                }
            }
        }

        public void ClearCell(int rowIndex, int ColIndex)
        {
            try
            {
                //The value row start with zero, so the row index should be > -1.
                if (rowIndex > -1 && ColIndex > 0 && m_paramGrid[ColIndex, rowIndex].Selected)
                {
                    m_paramGrid[ColIndex, rowIndex].ToolTipText = String.Empty;
                    if (m_paramGrid[ColIndex, rowIndex].Style.BackColor == ERROR_CELL_BACKGROUND)
                    {
                        m_paramGrid[ColIndex, rowIndex].Style.BackColor = DEFAULT_CELL_BACKGROUND;
                    }
                }
            }
            catch (ArgumentOutOfRangeException)
            {
                //When the row or column index is invalid
            }
            catch (Exception)
            {
                //Catch block for any other errors that are not handled
            }
        }

        #endregion





    }
}
