/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * VBTCodeCompressor.cs file acts as an utility file which holds the 
 * following class used in code formatting.
 * - CVBTCodeCompressor
 * - Node
 * - CVBTCodeGenWithOption
 * - PSetsSignalInfo
 * ------------------------------------------------------------------
 */

/*
 **************************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =============================================================================================
 *   Shoban Jayaraj 09/04/07    Initial Creation of Generic CVBTCodeCompressor & Node classes
 *   Suresh V A     09/10/07    Initial Creation of CVBTCodeGenWithOption & PSetsSignalInfo classes
 *   Suresh V A     09/14/07    Addressed IDC Review comments
 *   Suresh V A     09/18/07    Fix tersw00111443
 *   Suresh V A     10/10/07    Fix tersw00112567 ( Change the Reg Expression) 
 **************************************************************************************************
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    /// <summary>
    /// Given a fully qualified code snippet, this class optimizes the code by 
    /// introducing 'With/End With' blocks
    /// 
    /// Note: The compressor handles fully qualified code only. It will not 
    /// handle code that already has With / End With blocks.
    /// </summary>
    /// 
    /// <example>
    /// Compressor c = new Compressor();
    /// String compressedCode c.Compress(code);
    ///
    /// Where 'code' contains
    /// 
    /// TheHdw.Pins("Foo").ValueX = X
    /// TheHdw.Pins("Foo").ValueY = Y
    /// 
    /// after invoking, the following string is returned:
    /// 
    /// With TheHdw.Pins("Foo")
    ///     .ValueX = X
    ///     .ValueY = Y
    /// End With
    /// </example>
    class CVBTCodeCompressor
    {

        private const String ROOT_NODE = "root";

        /// <summary>
        /// Given a fully qualified VBA code snippet, optimized the code to use
        /// With / End With blocks
        /// </summary>
        /// <param name="code">The source code snippet. Can contain multiple lines 
        /// separated by newlines. Blanks and comments are preserved</param>
        /// <returns>Formatted string</returns>
        public String Compress(String code)
        {
            if (!String.IsNullOrEmpty(code))
            {
                Node n = Split(code);

                return Generate(n, 0, "", "");
            }
            return string.Empty;
        }

        /// <summary>
        /// Splits the spcified fully qualified source into tokens for optimization
        /// </summary>
        /// <param name="code">Fully qualifified VBA code snippet</param>
        /// <returns>The root node encompassing all tokens in a hierarchican manner</returns>
        private Node Split(String code)
        {
            Node root = new Node(ROOT_NODE);

            if (!String.IsNullOrEmpty(code))
            {
                String[] lines = code.Split('\n');

                foreach (String line in lines)
                {
                    String l = line.Trim();

                    Node n = root;

                    // handle comments or blank lines as nodes
                    if (l.Length == 0 || l.StartsWith("'"))
                    {
                        n.Append(l);
                    }
                    else
                    {
                        String[] segments = l.Split('.');

                        foreach (String segment in segments)
                        {
                            n = n.Append(segment.Trim());
                        }
                    }
                }
            }

            return root;
        }

        /// <summary>
        /// Given a root node, this method traverses the node list recursively and 
        /// generates optimized code removing redundant object calls with more efficient 
        /// With / End With blocks
        /// </summary>
        /// <param name="node">The root node provided by Split</param>
        /// <param name="level">Indicates the indentation level. This should be zero</param>
        /// <param name="history">The hierarchy of objects that has been processed so far. This can be an "" when called</param>
        /// <param name="generated">The generated code so far. This can be an ""</param>
        /// <returns>The optimized code organized using the With block</returns>
        private String Generate(Node node, int level, String history, String generated)
        {

            if (node != null)
            {
                LinkedList<Node> list = node.GetChildren();

                // If these are comments, then send them as is
                if (node.GetName().Length == 0 || node.GetName().StartsWith("'"))
                {
                    return generated + "".PadLeft(level, '\t') + node.GetName() + "\n";
                }

                if (!(level == 0 && node.GetName().Equals(ROOT_NODE))) history += "." + node.GetName();

                if (list.Count > 1 && node.GetName().IndexOf('(') != -1)
                {
                    String pad = "".PadLeft(level, '\t');

                    if (level == 0) history = history.Remove(0, 1); // remove the first dot
                    generated += pad + "With " + history + "\n";
                    foreach (Node n in list)
                    {
                        generated += Generate(n, level + 1, "", "");
                    }
                    generated += pad + "End With\n";
                }
                else if (list.Count >= 1)
                {
                    foreach (Node n in list)
                    {
                        generated += Generate(n, level, history, "");
                    }
                }
                else
                    generated += "".PadLeft(level, '\t') + history + "\n";

                return generated;
            }
            return string.Empty;
        }

        
    }

    /// <summary>
    /// A node represents a token (i.e, TheHdw, Pins("Foo") and so on). 
    /// 
    /// Newer child nodes can be Appended to an existing node.
    /// </summary>
    class Node
    {
        private String name;
        private LinkedList<Node> subNodes;

        public Node(String name)
        {
            this.name = name;
            subNodes = new LinkedList<Node>();
        }

        /// <summary>
        /// Appends a child node to it's internal list. 
        /// 
        /// An existing node is returned if it is the same as the last one that was added.
        /// </summary>
        /// <param name="name">The child node name</param>
        /// <returns>The node that has been created</returns>
        public Node Append(String name)
        {
            if (!String.IsNullOrEmpty(name))
            {
                // We allow duplicates and need to preserve the order
                Node last = null;
                if (subNodes.Last != null)
                    last = subNodes.Last.Value;

                if (last != null && last.GetName().Equals(name))
                    return last;

                Node n = new Node(name);
                subNodes.AddLast(n);

                return n;
            }
            return null;
        }

        public String GetName()
        {
            return name;
        }

        public LinkedList<Node> GetChildren()
        {
            return subNodes;
        }     
    }

    /// <summary>
    /// Enum is used to identifies what type of language
    /// </summary>
    internal enum TypeLang
    {
        PSET = 1,
        SOURCE_SIGNAL,
        CAPTURE_SIGNAL
    }

    class CVBTCodeGenWithOption
    {

        #region Constants

        private const string NEWLANG_STMT = "TheHdw.";
        public string NOTE_STMT = "'Note: Uncommenting a with block in a running function will            \nreset (stop) the VBA project.\n";
        private const string PIN = "Pin";
        private const string PSET = "PSets";
        private const string SOURCE_SIGNAL = "Source.Signals";
        private const string CAPTURE_SIGNAL = "Capture.Signals";



        ///Regular Expression for 
        ///TheHdw.[instrumentName].Pins("PinName").(PSets|(Source.Signals))("SignalName")
        private const string PATTERN_WITHOUT_USE_WITH_BLOCK = @"TheHdw\.(\w+)\.Pins\(\""(
(\w+)|(\w+(\,(\s*)\w+)*))\""\)\.(\w+|(\w+\.\w+))\(\""(\w+)\""\)";

        ///Regular Expression for 
        ///With TheHdw.[instrumentName].Pins("PinName").(PSets|(Source.Signals))("SignalName")
        private const string PATTERN_USE_WITH_BLOCK = @"((With\s+TheHdw\.(\w+)\.Pins\(\""(
(\w+)|(\w+(\,(\s*)\w+)*))\""\))|(With\s*))\.(\w+|(\w+\.\w+))\(\""(\w+)\""\)";

        ///Regular Expression for 
        ///With TheHdw.[instrumentName].Pins("PinName")
        ///         With (PSETS|Source.Signals|Capture.Signals)("[PinName]")
        private const string PATTERN_USE_WITH_BLOCK_PATTERN = @"(With\s+TheHdw\.(\w+)\.Pins\(\""(
(\w+)|(\w+(\,(\s*)\w+)*))\""\))|(With\s+)\.(\w+|(\w+\.\w+))\(\""(\w+)\""\)";



        #endregion Constants

        /// <summary>
        /// Used to get the VBT Code for Without using "With" 
        /// </summary>
        /// <param name="inputData">input string</param>
        /// <returns>returns the New language VBT code</returns>
        public string WithoutNewLanguageVBTCode(string inputData)
        {
            string indexstring = string.Empty;
            try
            {
                if (!String.IsNullOrEmpty(inputData))
                {
                    if (inputData.Contains("\n"))
                    {
                        List<PSetsSignalInfo> listPSetsSignals = GetPSetSignalsColl(inputData, PATTERN_WITHOUT_USE_WITH_BLOCK);
                        indexstring = FormNewLangStmt(listPSetsSignals);
                        inputData = inputData.Insert(0, indexstring);
                    }
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return inputData;
        }

        /// <summary>
        /// Gets the PSETS/SIGNAL Collection for Pinlanguage
        /// </summary>
        /// <param name="inputData">Pin language as input string</param>
        /// <param name="pattern">Pattern to match the item in the string</param>
        /// <returns>returns the PSETS/Signal Information as in the form of Collection</returns>
        private List<PSetsSignalInfo> GetPSetSignalsColl(string inputData, string pattern)
        {

            List<PSetsSignalInfo> listPSetsSignals = new List<PSetsSignalInfo>();
            try
            {
                Regex regExpWith = new Regex(pattern);
                Match matchedItem = regExpWith.Match(inputData); 

                //gets the PSETS/Signals Collection
                listPSetsSignals = GetCollection(inputData, pattern);

            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return listPSetsSignals;
        }

        /// <summary>
        /// Used to get the VBT Code for With using "With"
        /// </summary>
        /// <param name="inputdata">input string information</param>
        /// <returns>returns the new language VBT Code</returns>
        public string WithNewLanguageVBTCode(string inputdata)
        {
            string indexstring = string.Empty;       
            try
            {
                if (!String.IsNullOrEmpty(inputdata))
                {
                    if (inputdata.Contains("\n"))
                    {
                        List<PSetsSignalInfo> listPSetsSignals =
                                            GetPSetSignalInfoColl(inputdata, PATTERN_USE_WITH_BLOCK);
                        string formattedString = FormNewLangStmt(listPSetsSignals);
                        indexstring = NOTE_STMT + formattedString;
                        inputdata = inputdata.Insert(0, indexstring);
                        inputdata = GetCommentedOutput(inputdata);
                    }
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return inputdata;
        }

       
        /// <summary>
        /// Used to get the Collection of PsetInfo
        /// PSetInfo Contains the PinName and PSetsName property..
        /// Collection contains number of pins and pset info are present in the 
        /// VBT code
        /// </summary>
        /// <param name="inputData">input data </param>
        /// <param name="pattern">Pattern need to match </param>
        /// <returns>returns the PSets/Signals Collection</returns>     
        private List<PSetsSignalInfo> GetPSetSignalInfoColl(string inputData, string pattern)
        {
            List<PSetsSignalInfo> listPSetsSignals = new List<PSetsSignalInfo>();
            try
            {
                Regex regExpWith = new Regex(pattern);
                Match matchedItem = regExpWith.Match(inputData); ;
                if (GetDotCount(matchedItem.Value) <= 2)
                {
                    ///With TheHdw.[InstrumentName].Pins("[PinName]")
                    ///                         With .PSets("[PSetName]")
                    listPSetsSignals = GetCollection(inputData, PATTERN_USE_WITH_BLOCK_PATTERN);
                }
                else
                {
                    ///With TheHdw.[InstrumentName].Pins("[PinName]").PSets("[PSetName]")
                    listPSetsSignals = GetCollection(inputData, PATTERN_USE_WITH_BLOCK);
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return listPSetsSignals;
        }

        /// <summary>
        /// Used to Get the Collection PSets/Signals Collection
        /// </summary>
        /// <param name="inputData">input data as a string</param>
        /// <param name="pattern">Regular expression string used to match the Item in a string</param>
        /// <returns>returns the PSETS/Signals collection</returns>
        private List<PSetsSignalInfo> GetCollection(string inputData, string pattern)
        {
            string pinName = string.Empty;
            string pSetName = string.Empty;
            string signalName = string.Empty;
            string instrumentName = string.Empty;

            List<PSetsSignalInfo> listPSetsSignals = new List<PSetsSignalInfo>();
            try
            {

                Regex regExpWith = new Regex(pattern);
                MatchCollection matchCollection = regExpWith.Matches(inputData);
                if (matchCollection != null)
                {
                    foreach (Match matchItem in matchCollection)
                    {
                        PSetsSignalInfo pSetsSignalInfo = null;
                        //Used to get the pinName.
                        if (matchItem.Value != null)
                        {
                            if (matchItem.Value.Contains(PIN))
                            {
                                pinName = GetPinName(matchItem);
                                instrumentName = GetInstrumentName(matchItem);
                            }
            

                            if (matchItem.Value.Contains(PSET))
                            {
                                //Used to get the PSets Name
                                pSetName = GetPSetsSignalName(matchItem);

                                //The Pinname and PSetName are added into the Collection                    
                                pSetsSignalInfo = new PSetsSignalInfo(pinName, instrumentName, pSetName, signalName, TypeLang.PSET);
                                listPSetsSignals.Add(pSetsSignalInfo);
                            }
                            else if (matchItem.Value.Contains(SOURCE_SIGNAL))
                            {
                                //Used to get the Signal Name
                                signalName = GetPSetsSignalName(matchItem);

                                //The Pinname and PSetName are added into the Collection                    
                                pSetsSignalInfo = new PSetsSignalInfo(pinName, instrumentName, pSetName, signalName, TypeLang.SOURCE_SIGNAL);
                                listPSetsSignals.Add(pSetsSignalInfo);
                            }
                            else if (matchItem.Value.Contains(CAPTURE_SIGNAL))
                            {
                                //Used to get the Signal Name
                                signalName = GetPSetsSignalName(matchItem);

                                //The Pinname and PSetName are added into the Collection                    
                                pSetsSignalInfo = new PSetsSignalInfo(pinName, instrumentName, pSetName, signalName, TypeLang.CAPTURE_SIGNAL);
                                listPSetsSignals.Add(pSetsSignalInfo);
                            }
                        }
                    }
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return listPSetsSignals;
        }

           

        /// <summary>
        /// used to get the no of dots in a match Item..
        /// coz, with out checking dots..will get the
        /// ArgumentNullException.when doing the string manipulation.
        /// This function is for used to find the "NESTED WITH BLOCK"...
        /// </summary>
        /// <param name="value">input data in the form of string</param>
        /// <returns>returns  the count</returns>
        private int GetDotCount(string value)
        {
            int nDotCount = 0;
            try
            {
                while (value.Contains("."))
                {
                    value = value.Remove(0, value.IndexOf('.') + 1).Trim();
                    nDotCount++;
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return nDotCount;
        }

        /// <summary>
        /// This method is used to get the Instrumentname from the 
        /// Match Item
        /// </summary>
        /// <param name="matchItem"></param>
        /// <returns>returns the Instrument Name</returns>
        private string GetInstrumentName(Match matchItem)
        {
            string instrumentName = string.Empty;
            string matchItemVal = string.Empty;
            try
            {
                if (matchItem.Value != null)
                {
                    if (matchItem.Value.Contains(PIN))
                    {
                        matchItemVal = matchItem.Value.Remove(0, matchItem.Value.IndexOf('.') + 1).Trim();
                        instrumentName = matchItemVal.Substring(0, matchItemVal.IndexOf('.')).Trim();
                    }
                }
            }
            catch (Exception exp)
            {
                Trace.Write(exp.Message);
            }
            return instrumentName;
        }

        /// <summary>
        /// Used to get the Pinname
        /// </summary>
        /// <param name="matchItem">Match Item</param>
        /// <returns>returns the Pinname</returns>
        private string GetPinName(Match matchItem)
        {
            string pinName = string.Empty;
            string matchItemInfo = string.Empty;
            string searchPinName = string.Empty;
            int nstartIndex = 0, nEndIndex = 0;
            try
            {
                if (matchItem != null)
                {
                    matchItemInfo = matchItem.Value;
                    nstartIndex = matchItemInfo.IndexOf("\"");
                    nEndIndex = matchItemInfo.IndexOf(')');
                    pinName = matchItemInfo.Substring(nstartIndex, (nEndIndex - nstartIndex));
                    pinName = pinName.Replace("\"", " ").Trim();                    
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return pinName;
        }

        /// <summary>
        /// Used to get the PsetName
        /// </summary>
        /// <param name="matchItem">Match Item</param>
        /// <returns>returns the PSetname</returns>
        private string GetPSetsSignalName(Match matchItem)
        {
            string pSetSignalName = string.Empty;
            string matchItemInfo = string.Empty;
            string searchPinName = string.Empty;
            int nstartIndex = 0, nEndIndex = 0;
            try
            {
                if (matchItem != null)
                {
                    if (matchItem.Value.Contains(PIN))
                        matchItemInfo = matchItem.Value.Remove(0, matchItem.Value.IndexOf(")") + 2).Trim();
                    else
                        matchItemInfo = matchItem.Value.Remove(0, matchItem.Value.IndexOf('.'));

                    nstartIndex = matchItemInfo.IndexOf("\"");
                    nEndIndex = matchItemInfo.IndexOf(')');
                    pSetSignalName = matchItemInfo.Substring(nstartIndex, (nEndIndex - nstartIndex));
                    pSetSignalName = pSetSignalName.Replace("\"", " ").Trim();                  
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return pSetSignalName;
        }

        /// <summary>
        /// used to generate the stmt which we need to add in the Header
        /// </summary>
        /// <param name="psetInfoCol"></param>
        /// <returns></returns>
        private string FormNewLangStmt(List<PSetsSignalInfo> psetsigInfoCol)
        {
            string formattedData = string.Empty;
            string tempRetData = string.Empty;
            List<string> listFormattedData = new List<string>();
            try
            {
                if (psetsigInfoCol != null)
                {
                    foreach (PSetsSignalInfo pSetsSignalInfoItem in psetsigInfoCol)
                    {
                        switch (pSetsSignalInfoItem.TypeLanguage)
                        {
                            case TypeLang.PSET:
                                formattedData = NEWLANG_STMT + pSetsSignalInfoItem.InstrumentName + ".Pins(\"" + pSetsSignalInfoItem.PinName + "\")" + ".Psets.Add \"" + pSetsSignalInfoItem.PSetsName + "\"\n";
                                break;
                            case TypeLang.SOURCE_SIGNAL:
                                formattedData = NEWLANG_STMT + pSetsSignalInfoItem.InstrumentName + ".Pins(\"" + pSetsSignalInfoItem.PinName + "\")" + ".Source.Signals.Add \"" + pSetsSignalInfoItem.SignalName + "\"\n";
                                break;
                            case TypeLang.CAPTURE_SIGNAL:
                                formattedData = NEWLANG_STMT + pSetsSignalInfoItem.InstrumentName + ".Pins(\"" + pSetsSignalInfoItem.PinName + "\")" + ".Capture.Signals.Add \"" + pSetsSignalInfoItem.SignalName + "\"\n";
                                break;
                        }

                        ///Check The Formstring is already present in the collection or not
                        if (!CheckFormStringAreAvailable(listFormattedData, formattedData))
                            listFormattedData.Add(formattedData);


                    }
                    //Concat string collection into Single String.
                    tempRetData = ConcateStringCollection(listFormattedData);
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return tempRetData;
        }

        /// <summary>
        /// Check whether the string already present in the list or not...
        /// </summary>
        /// <param name="formattedstringCollection">input as a Formatted Collection</param>
        /// <param name="formString">formatted Data as a Input</param>
        /// <returns>returns the boolean value</returns>
        private bool CheckFormStringAreAvailable(List<String> listFormattedData, string formattedData)
        {
            try
            {
                foreach (String formStrItem in listFormattedData)
                {
                    if (formStrItem.Equals(formattedData))
                        return true;
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return false;
        }


        /// <summary>
        /// Concate all the strings in String Collection
        /// </summary>
        /// <param name="strData">String Collection</param>
        /// <returns>returns the Concatenated String</returns>
        private string ConcateStringCollection(List<string> listFormattedData)
        {
            string tempData = string.Empty;
            try
            {
                if (listFormattedData != null)
                {
                    if (listFormattedData.Count > 0)
                    {
                        foreach (string strItem in listFormattedData)
                        {
                            tempData += strItem;
                        }
                    }
                    return tempData;
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return string.Empty;
        }

       /// <summary>
        /// Used to comment all the Stmts
        /// </summary>
        /// <param name="inputValue">input data in the form of string</param>
        /// <returns>returns the commented output</returns>
        public string GetCommentedOutput(string inputValue)
        {
            string data = string.Empty;
            try
            {

                if (!string.IsNullOrEmpty(inputValue))
                {
                    string delimiter = "\n\t";
                    char[] indexedValue = delimiter.ToCharArray();

                    data = inputValue.TrimEnd(indexedValue);
                    data = data.Replace("\n\n", "\n");
                    data = data.Replace("\n", "\n ' ");                  
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return data;
        }

        /// <summary>
        /// Used to add with block comment
        /// </summary>
        /// <param name="inputValue">input data in the form of string</param>
        /// <returns>returns the output with comment added</returns>
        public string AddWithBlockComment(string inputValue)
        {
            string data = string.Empty;
            try
            {
                if (!string.IsNullOrEmpty(inputValue))
                {
                    data = NOTE_STMT + inputValue;
                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
            return data;
        }
    }


    /// <summary>
    /// Purpose: Used to add the TheHdw.[InstrumentName].Pins(["PinName"]).PSets.Add "[PsetName]"
    /// </summary>
    class PSetsSignalInfo
    {
        
        private string m_pSetName;

        private string m_SignalName;

        private string m_InstrumentName;

        private string m_pinName;

        private TypeLang m_TypeLang;


        public PSetsSignalInfo(string pinName, string instrumentName, string pSetName, string signalName, TypeLang typeLang)
        {
            m_pinName = pinName;
            m_InstrumentName = instrumentName;
            m_pSetName = pSetName;
            m_SignalName = signalName;
            m_TypeLang = typeLang;
        }

        /// <summary>
        /// Get the type of the Pin language [PSETS/Sorce.Signals/Capture.Signals]
        /// </summary>
        public TypeLang TypeLanguage
        {
            get { return m_TypeLang; }

        }

        /// <summary>
        /// Get and Set the PsetName
        /// </summary>
        public string PSetsName
        {
            get { return m_pSetName; }
           
        }

        /// <summary>
        /// Get and Set the Signalname
        /// </summary>
        public string SignalName
        {
            get { return m_SignalName; }
           
        }

        /// <summary>
        /// Get and Set the Instrument Name
        /// </summary>
        public string InstrumentName
        {
            get { return m_InstrumentName; }
           
        }


        /// <summary>
        /// Get and Set the PSetName
        /// </summary>
        public string PinName
        {
            get { return m_pinName; }
          
        }
    }
}
